@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'ffi-glib/container_class_methods'
 require 'ffi-glib/array_methods'
 
@@ -13,14 +14,25 @@ module GLib
 
     attr_reader :element_type
 
-    class << self; undef :new; end
-
-    def self.new type
-      ptr = Lib.g_array_new(0, 0, calculated_element_size(type))
-      wrap type, ptr
+    def initialize(type)
+      @element_type = type
+      ptr = Lib.g_array_new(0, 0, calculated_element_size)
+      store_pointer(ptr)
     end
 
-    def append_vals ary
+    # @api private
+    def self.from_enumerable(elmtype, it)
+      new(elmtype).tap { |arr| arr.append_vals it }
+    end
+
+    # @api private
+    def self.calculated_element_size(type)
+      ffi_type = GirFFI::TypeMap.type_specification_to_ffi_type(type)
+      FFI.type_size(ffi_type)
+    end
+
+    # @override
+    def append_vals(ary)
       bytes = GirFFI::InPointer.from_array element_type, ary
       Lib.g_array_append_vals(self, bytes, ary.length)
       self
@@ -36,21 +48,18 @@ module GLib
       @struct[:len]
     end
 
-    def data_ptr
-      @struct[:data]
-    end
-
     def get_element_size
       Lib.g_array_get_element_size self
     end
 
     alias_method :element_size, :get_element_size
 
-    def == other
+    def ==(other)
       to_a == other.to_a
     end
 
-    def reset_typespec typespec = nil
+    # @api private
+    def reset_typespec(typespec = nil)
       if typespec
         @element_type = typespec
         check_element_size_match
@@ -60,15 +69,10 @@ module GLib
       self
     end
 
-    def self.from_enumerable elmtype, it
-      new(elmtype).tap { |arr| arr.append_vals it }
-    end
-
     private
 
-    def self.calculated_element_size type
-      ffi_type = GirFFI::TypeMap.type_specification_to_ffitype(type)
-      FFI.type_size(ffi_type)
+    def data_ptr
+      @struct[:data]
     end
 
     def calculated_element_size
@@ -77,7 +81,7 @@ module GLib
 
     def check_element_size_match
       unless calculated_element_size == get_element_size
-        warn "WARNING: Element sizes do not match"
+        warn 'WARNING: Element sizes do not match'
       end
     end
 
