@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 GLib.load_class :Bytes
 
 module GLib
@@ -7,20 +8,21 @@ module GLib
 
     remove_method :get_data if method_defined? :get_data
 
-    # Override for GBytes#get_data, needed due to mis-identification of the
-    # element-type of the resulting sized array.
+    # @override
+    # NOTE: Needed due to mis-identification of the element-type of the
+    # resulting sized array for the default binding.
     def get_data
-      length_ptr = GirFFI::InOutPointer.for :gsize
+      length_ptr = FFI::MemoryPointer.new :size_t
       data_ptr = Lib.g_bytes_get_data self, length_ptr
-      length = length_ptr.to_value
+      length = length_ptr.get_size_t(0)
       GirFFI::SizedArray.wrap(:guint8, length, data_ptr)
     end
 
-    def each &block
+    def each(&block)
       data.each(&block)
     end
 
-    def self.from it
+    def self.from(it)
       case it
       when self
         it
@@ -31,13 +33,9 @@ module GLib
       end
     end
 
-    class << self
-      undef new
-    end
-
-    def self.new arr
+    def initialize(arr)
       data = GirFFI::SizedArray.from :guint8, arr.size, arr
-      wrap Lib.g_bytes_new data.to_ptr, data.size
+      store_pointer Lib.g_bytes_new data.to_ptr, data.size
     end
 
     private
