@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 module GLib
   # Common methods for container classes: Array, PtrArray, List, SList and
   # HashTable.
@@ -9,6 +10,8 @@ module GLib
               nil
             when FFI::Pointer
               ptr
+            when self
+              ptr.to_ptr
             when GirFFI::BoxedBase
               ptr.to_ptr
             end
