@@ -1,16 +1,16 @@
+# frozen_string_literal: true
 GLib.load_class :Error
 
 module GLib
   # Overrides for GError, used by GLib for handling non-fatal errors.
   class Error
-    # TODO: Auto-convert strings and symbols to quarks
-    GIR_FFI_DOMAIN = GLib.quark_from_string("gir_ffi")
+    GIR_FFI_DOMAIN = GLib.quark_from_string('gir_ffi')
 
-    def self.from_exception ex
+    def self.from_exception(ex)
       new_literal GIR_FFI_DOMAIN, 0, ex.message
     end
 
-    def self.from it
+    def self.from(it)
       from_exception it
     end
   end
