@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'ffi-glib/container_class_methods'
 
 GLib.load_class :HashTable
@@ -11,12 +12,28 @@ module GLib
     attr_reader :key_type
     attr_reader :value_type
 
+    def initialize(key_type, value_type)
+      @key_type = key_type
+      @value_type = value_type
+      store_pointer Lib.g_hash_table_new(
+        hash_function_for_key_type, equality_function_for_key_type)
+    end
+
+    # @api private
+    def self.from_enumerable(typespec, hash)
+      ghash = new(*typespec)
+      hash.each do |key, val|
+        ghash.insert key, val
+      end
+      ghash
+    end
+
     def each
-      prc = proc {|keyptr, valptr, _userdata|
+      prc = proc do |keyptr, valptr, _userdata|
         key = GirFFI::ArgHelper.cast_from_pointer key_type, keyptr
         val = GirFFI::ArgHelper.cast_from_pointer value_type, valptr
         yield key, val
-      }
+      end
       callback = GLib::HFunc.from prc
       ::GLib::Lib.g_hash_table_foreach to_ptr, callback, nil
     end
@@ -25,55 +42,40 @@ module GLib
       Hash[to_a]
     end
 
-    def insert key, value
+    # @override
+    def insert(key, value)
       keyptr = GirFFI::InPointer.from key_type, key
       valptr = GirFFI::InPointer.from value_type, value
       ::GLib::Lib.g_hash_table_insert to_ptr, keyptr, valptr
     end
 
-    class << self; remove_method :new; end
-
-    def self.new keytype, valtype
-      wrap [keytype, valtype], Lib.g_hash_table_new(
-        hash_function_for(keytype), equality_function_for(keytype))
-    end
-
-    def reset_typespec typespec
+    # @api private
+    def reset_typespec(typespec)
       @key_type, @value_type = *typespec
       self
     end
 
-    def self.from_enumerable typespec, hash
-      ghash = new(*typespec)
-      hash.each do |key, val|
-        ghash.insert key, val
-      end
-      ghash
-    end
+    private
 
-    def self.hash_function_for keytype
-      case keytype
+    def hash_function_for_key_type
+      case @key_type
       when :utf8
         FFI::Function.new(:uint,
                           [:pointer],
-                          find_support_function("g_str_hash"))
-      else
-        nil
+                          find_support_function('g_str_hash'))
       end
     end
 
-    def self.equality_function_for keytype
-      case keytype
+    def equality_function_for_key_type
+      case @key_type
       when :utf8
         FFI::Function.new(:int,
                           [:pointer, :pointer],
-                          find_support_function("g_str_equal"))
-      else
-        nil
+                          find_support_function('g_str_equal'))
       end
     end
 
-    def self.find_support_function name
+    def find_support_function(name)
       lib = ::GLib::Lib.ffi_libraries.first
       lib.find_function(name)
     end
