@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'ffi-glib/list_methods'
 
 GLib.load_class :List
@@ -7,11 +8,11 @@ module GLib
   class List
     include ListMethods
 
-    def self.from_enumerable type, arr
-      arr.inject(new type) { |lst, val| lst.append val }
+    def self.from_enumerable(type, arr)
+      arr.reduce(new(type)) { |lst, val| lst.append val }
     end
 
-    def append data
+    def append(data)
       self.class.wrap(element_type,
                       Lib.g_list_append(self, element_ptr_for(data)))
     end
