@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'singleton'
 
 GLib.load_class :MainLoop
@@ -26,7 +27,7 @@ module GLib
 
       def handler_proc
         proc do
-          Thread.pass
+          ::Thread.pass
           true
         end
       end
@@ -54,15 +55,16 @@ module GLib
 
     def self.handle_exception(ex)
       current_loop = RUNNING_LOOPS.last
-      if current_loop
-        EXCEPTIONS << ex
-        current_loop.quit
-      else
-        raise ex
-      end
+      raise ex unless current_loop
+      EXCEPTIONS << ex
+      current_loop.quit
     end
 
     alias_method :run_without_thread_enabler, :run
     alias_method :run, :run_with_thread_enabler
   end
 end
+
+Signal.trap 'INT' do
+  GLib::MainLoop.handle_exception(Interrupt.new)
+end
