@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'ffi-glib/list_methods'
 
 GLib.load_class :SList
@@ -8,7 +9,7 @@ module GLib
     include ListMethods
 
     def self.from_enumerable(type, arr)
-      arr.reverse.reduce(new type) { |lst, val| lst.prepend val }
+      arr.reverse.reduce(new(type)) { |lst, val| lst.prepend val }
     end
 
     def prepend(data)
