@@ -1,29 +1,34 @@
+# frozen_string_literal: true
 require 'gir_ffi/core'
 
-GirFFI.setup :GObject
-
-require 'ffi-gobject/base'
+# Bypass check for existing modules
+GirFFI::Builders::ModuleBuilder.new('GObject').generate
 
 require 'ffi-gobject/value'
 require 'ffi-gobject/initially_unowned'
 require 'ffi-gobject/closure'
 require 'ffi-gobject/object'
 require 'ffi-gobject/object_class'
+require 'ffi-gobject/param_spec'
 require 'ffi-gobject/ruby_closure'
-require 'gir_ffi/builders/user_defined_builder'
+require 'gir_ffi/signal_not_found_error'
 
 # Module representing GLib's GObject namespace.
 module GObject
   def self.type_from_instance_pointer(inst_ptr)
     return nil if inst_ptr.null?
     klsptr = inst_ptr.get_pointer 0
-    GirFFI::InOutPointer.new(:GType, klsptr).to_value
+    klsptr.get_gtype 0
   end
 
   def self.type_from_instance(instance)
     type_from_instance_pointer instance.to_ptr
   end
 
+  def self.type_name_from_instance(instance)
+    type_name type_from_instance instance
+  end
+
   def self.object_class_from_instance(instance)
     object_class_from_instance_pointer instance.to_ptr
   end
@@ -70,14 +75,6 @@ module GObject
     signal_connect object, detailed_signal, data, true, &block
   end
 
-  # Smells of :reek:LongParameterList: due to the C interface.
-  # rubocop:disable Metrics/ParameterLists
-  def self.param_spec_int(name, nick, blurb, minimum, maximum, default_value, flags)
-    ptr = Lib.g_param_spec_int(name, nick, blurb, minimum, maximum,
-                               default_value, flags)
-    ParamSpecInt.wrap(ptr)
-  end
-
   load_class :Callback
   load_class :ClosureNotify
   load_class :ConnectFlags
@@ -90,6 +87,8 @@ module GObject
     attach_function :g_object_ref, [:pointer], :pointer
     attach_function :g_object_unref, [:pointer], :pointer
 
+    attach_function :g_value_copy, [:pointer, :pointer], :void
+    attach_function :g_value_init, [:pointer, :size_t], :pointer
     attach_function :g_value_unset, [:pointer], :pointer
 
     attach_function :g_array_get_type, [], :size_t
@@ -100,12 +99,16 @@ module GObject
     attach_function :g_signal_connect_data,
                     [:pointer, :string, Callback, :pointer, ClosureNotify, ConnectFlags],
                     :ulong
+
+    attach_function :g_closure_ref, [:pointer], :pointer
+    attach_function :g_closure_sink, [:pointer], :pointer
     attach_function :g_closure_set_marshal,
                     [:pointer, ClosureMarshal], :void
 
-    attach_function :g_param_spec_int,
-                    [:string, :string, :string, :int32, :int32, :int32, ParamFlags],
-                    :pointer
+    attach_function :g_param_spec_ref, [:pointer], :pointer
+    attach_function :g_param_spec_sink, [:pointer], :pointer
+
+    attach_function :g_type_class_ref, [:size_t], :pointer
   end
 
   TYPE_ARRAY = Lib.g_array_get_type
