@@ -1,3 +1,6 @@
+# frozen_string_literal: true
+require 'gir_ffi/property_not_found_error'
+
 GObject.load_class :Object
 
 module GObject
@@ -8,9 +11,7 @@ module GObject
     def initialize_with_automatic_gtype(properties = {})
       gparameters = properties.map do |name, value|
         name = name.to_s
-        unless property_param_spec(name)
-          raise ArgumentError, "Property '#{name}' not found in class #{self.class}"
-        end
+        property_param_spec(name)
         GObject::Parameter.new.tap do |gparam|
           gparam.name = name
           gparam.value = value
@@ -25,16 +26,22 @@ module GObject
 
     private :base_initialize
 
-    def store_pointer(ptr)
-      super
-      klass = self.class
-      ObjectSpace.define_finalizer self, klass.make_finalizer(ptr, klass.name)
+    remove_method :ref
+
+    def ref
+      Lib.g_object_ref self
+      self
     end
 
-    def self.make_finalizer(ptr, name)
+    def store_pointer(ptr)
+      super
+      ObjectSpace.define_finalizer self, self.class.make_finalizer(ptr)
+    end
+
+    def self.make_finalizer(ptr)
       proc do
         rc = GObject::Object::Struct.new(ptr)[:ref_count]
-        if rc == 0
+        if rc.zero?
           warn "not unreffing #{name}:#{ptr} (#{rc})"
         else
           GObject::Lib.g_object_unref ptr
@@ -53,6 +60,10 @@ module GObject
       super
     end
 
+    def respond_to_missing?(*)
+      false
+    end
+
     def signal_connect(event, data = nil, &block)
       GObject.signal_connect(self, event, data, &block)
     end
@@ -67,12 +78,7 @@ module GObject
     def get_property_extended(property_name)
       value = get_property(property_name)
       type_info = get_property_type property_name
-      case type_info.tag
-      when :ghash, :glist
-        adjust_value_to_type value, type_info
-      else
-        value
-      end
+      property_value_post_conversion value, type_info
     end
 
     def get_property_with_override(property_name)
@@ -83,8 +89,7 @@ module GObject
 
     def set_property_extended(property_name, value)
       type_info = get_property_type property_name
-      adjusted_value = adjust_value_to_type(value, type_info)
-
+      adjusted_value = property_value_pre_conversion(value, type_info)
       set_property property_name, adjusted_value
     end
 
@@ -106,7 +111,7 @@ module GObject
     private
 
     def get_property_type(property_name)
-      prop = self.class.find_property property_name
+      prop = self.class.find_property(property_name)
       prop.property_type
     end
 
@@ -116,15 +121,31 @@ module GObject
     end
 
     def property_gtype(property_name)
-      property_param_spec(property_name).value_type
+      pspec = property_param_spec(property_name)
+      pspec.value_type
     end
 
     def property_param_spec(property_name)
-      object_class.find_property property_name
+      object_class.find_property property_name or
+        raise GirFFI::PropertyNotFoundError.new(property_name, self.class)
     end
 
-    # TODO: Move to ITypeInfo
-    def adjust_value_to_type(val, type_info)
+    # TODO: Move to ITypeInfo and unify with ArgHelper.cast_from_pointer
+    def property_value_post_conversion(val, type_info)
+      case type_info.flattened_tag
+      when :ghash
+        GLib::HashTable.from type_info.element_type, val
+      when :glist
+        GLib::List.from type_info.element_type, val
+      when :callback
+        GirFFI::Builder.build_class(type_info.interface).wrap val
+      else
+        val
+      end
+    end
+
+    # TODO: Move to ITypeInfo and unify with ArgHelper.cast_from_pointer
+    def property_value_pre_conversion(val, type_info)
       case type_info.flattened_tag
       when :ghash
         GLib::HashTable.from type_info.element_type, val
@@ -132,6 +153,8 @@ module GObject
         GLib::List.from type_info.element_type, val
       when :strv
         GLib::Strv.from val
+      when :callback
+        GirFFI::Builder.build_class(type_info.interface).from val
       else
         val
       end
