@@ -1,23 +1,31 @@
+# frozen_string_literal: true
 GObject.load_class :Value
 
 module GObject
   # Overrides for GValue, GObject's generic value container structure.
   class Value
-    setup_instance_method 'init'
+    remove_method :init
 
-    def init_with_finalizer(type)
-      return self if [TYPE_NONE, TYPE_INVALID].include? type
-      init_without_finalizer(type)
-      ObjectSpace.define_finalizer self, self.class.make_finalizer(to_ptr)
+    def initialize
+      super
+      struct.owned = true
+      to_ptr.autorelease = nil
+    end
+
+    def init(type)
+      Lib.g_value_init self, type unless [TYPE_NONE, TYPE_INVALID].include? type
       self
     end
 
-    alias_method :init_without_finalizer, :init
-    alias_method :init, :init_with_finalizer
-
-    def self.make_finalizer(ptr)
+    def self.make_finalizer(struct)
       proc do
-        GObject::Lib.g_value_unset ptr
+        if struct.owned?
+          ptr = struct.to_ptr
+          unless struct[:g_type] == TYPE_INVALID
+            Lib.g_value_unset ptr
+          end
+          GObject.boxed_free gtype, ptr
+        end
       end
     end
 
@@ -45,7 +53,7 @@ module GObject
       TYPE_OBJECT    => [:get_object,        :set_instance_enhanced],
       TYPE_GTYPE     => [:get_gtype,         :set_gtype],
       TYPE_VARIANT   => [:get_variant,       :set_variant]
-    }
+    }.freeze
 
     def set_value(val)
       send set_method, val
@@ -106,6 +114,16 @@ module GObject
       end
     end
 
+    def self.copy_value_to_pointer(value, pointer, offset = 0)
+      target = wrap(pointer + offset)
+      target.init(value.current_gtype)
+      Lib.g_value_copy value, target unless value.uninitialized?
+    end
+
+    def uninitialized?
+      current_gtype == TYPE_INVALID
+    end
+
     private
 
     def set_ruby_value(val)
@@ -119,7 +137,7 @@ module GObject
       FalseClass => TYPE_BOOLEAN,
       Integer => TYPE_INT,
       String => TYPE_STRING
-    }
+    }.freeze
 
     def init_for_ruby_value(val)
       if val.class.respond_to? :gtype
@@ -137,10 +155,6 @@ module GObject
     def get_none
     end
 
-    def uninitialized?
-      current_gtype == TYPE_INVALID
-    end
-
     def set_instance_enhanced(val)
       check_type_compatibility val if val
       set_instance val
