@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'ffi-gobject_introspection/i_base_info'
 module GObjectIntrospection
   # Wraps a GIArgInfo struct.
@@ -44,8 +45,7 @@ module GObjectIntrospection
     end
 
     def argument_type
-      ITypeInfo.wrap(Lib.g_arg_info_get_type @gobj)
+      ITypeInfo.wrap Lib.g_arg_info_get_type(@gobj)
     end
   end
 end
-
