@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 module GObjectIntrospection
   # Wraps a GIConstantInfo struct; represents a constant.
   class IConstantInfo < IBaseInfo
@@ -13,21 +14,21 @@ module GObjectIntrospection
       guint64:  :v_uint64,
       gdouble:  :v_double,
       utf8:     :v_string
-    }
+    }.freeze
 
     def value
       case type_tag
       when :utf8
         raw_value.force_encoding('utf-8')
       when :gboolean
-        raw_value != 0
+        raw_value.nonzero? ? true : false
       else
         raw_value
       end
     end
 
     def constant_type
-      ITypeInfo.wrap(Lib.g_constant_info_get_type @gobj)
+      ITypeInfo.wrap Lib.g_constant_info_get_type self
     end
 
     private
@@ -38,7 +39,7 @@ module GObjectIntrospection
 
     def raw_value
       value_union = Lib::GIArgument.new
-      Lib.g_constant_info_get_value @gobj, value_union
+      Lib.g_constant_info_get_value self, value_union
       value_union[union_member_key]
     end
 
