@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 module GObjectIntrospection
   # Wraps a GIRegisteredTypeInfo struct.
   # Represents a registered type.
@@ -13,5 +14,7 @@ module GObjectIntrospection
     def g_type
       Lib.g_registered_type_info_get_g_type @gobj
     end
+
+    alias_method :gtype, :g_type
   end
 end
