@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 module GObjectIntrospection
   # Wraps a GITypeInfo struct.
   # Represents type information, direction, transfer etc.
@@ -11,7 +12,7 @@ module GObjectIntrospection
     end
 
     def param_type(index)
-      ITypeInfo.wrap(Lib.g_type_info_get_param_type @gobj, index)
+      ITypeInfo.wrap Lib.g_type_info_get_param_type(@gobj, index)
     end
 
     def interface
