@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 module GObjectIntrospection
   # Wraps a GIUnionInfo struct.
   # Represents a union.
@@ -6,8 +7,8 @@ module GObjectIntrospection
       Lib.g_union_info_get_n_fields @gobj
     end
 
-    def field index
-      IFieldInfo.wrap(Lib.g_union_info_get_field @gobj, index)
+    def field(index)
+      IFieldInfo.wrap Lib.g_union_info_get_field(@gobj, index)
     end
 
     ##
@@ -17,15 +18,15 @@ module GObjectIntrospection
       Lib.g_union_info_get_n_methods @gobj
     end
 
-    def get_method index
-      IFunctionInfo.wrap(Lib.g_union_info_get_method @gobj, index)
+    def get_method(index)
+      IFunctionInfo.wrap Lib.g_union_info_get_method(@gobj, index)
     end
 
     ##
     build_array_method :get_methods
 
-    def find_method name
-      IFunctionInfo.wrap(Lib.g_union_info_find_method @gobj, name)
+    def find_method(name)
+      IFunctionInfo.wrap Lib.g_union_info_find_method(@gobj, name.to_s)
     end
 
     def size
