@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'gir_ffi/builders/type_builder'
 require 'gir_ffi/builders/module_builder'
 require 'gir_ffi/builder_helper'
@@ -28,7 +29,16 @@ module GirFFI
     end
 
     def self.build_module(namespace, version = nil)
-      Builders::ModuleBuilder.new(namespace, version).generate
+      module_name = namespace.sub(/\A./, &:upcase)
+      if const_defined? module_name
+        modul = const_get module_name
+        unless modul.const_defined? :GIR_FFI_BUILDER
+          raise "The module #{module_name} was already defined elsewhere"
+        end
+      end
+      Builders::ModuleBuilder.new(module_name,
+                                  namespace: namespace,
+                                  version: version).generate
     end
 
     # TODO: Move elsewhere, perhaps to FunctionBuilder.
