@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 module GirFFI
   module Builders
     # Class representing the argument and return value builders for a method,
@@ -36,18 +37,10 @@ module GirFFI
       def method_argument_names
         @method_argument_names ||=
           begin
-            base = []
-            block = nil
-            argument_builders.each do |it|
-              name = it.method_argument_name
-              if !block && it.block_argument?
-                block = "&#{name}"
-              else
-                base << name
-              end
-            end
-            base << block if block
-            base.compact
+            base, block = split_off_block_argument
+            args = base_argument_names(base)
+            args << "&#{block.method_argument_name}" if block
+            args
           end
       end
 
@@ -117,6 +110,25 @@ module GirFFI
         @sorted_base_argument_builders ||= @base_argument_builders.
           sort_by.with_index { |arg, i| [arg.array_length_idx, i] }
       end
+
+      def split_off_block_argument
+        builders_with_name = argument_builders.select(&:method_argument_name)
+        blocks, base = builders_with_name.partition(&:block_argument?)
+        return base, blocks.first
+      end
+
+      def base_argument_names(arguments)
+        required_found = false
+        arguments.reverse.map do |it|
+          name = it.method_argument_name
+          if it.allow_none? && !required_found
+            "#{name} = nil"
+          else
+            required_found = true
+            name
+          end
+        end.compact.reverse
+      end
     end
   end
 end
