@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 module GirFFI
   module Builders
     # Abstract parent class of the argument building classes. These
@@ -10,7 +11,7 @@ module GirFFI
         module next nil not or redo rescue retry
         return self super then true undef unless
         until when while yield
-      )
+      ).freeze
 
       attr_reader :arginfo
       attr_accessor :length_arg, :array_arg
@@ -37,7 +38,6 @@ module GirFFI
       end
 
       def specialized_type_tag
-        # SMELL: Law of Demeter, due to this being arginfo.argument_type.flattened_tag
         type_info.flattened_tag
       end
 
