@@ -1,24 +1,26 @@
+# frozen_string_literal: true
 require 'gir_ffi/builder_helper'
 
 module GirFFI
-
   # Base class for type builders.
   class BaseTypeBuilder
     include BuilderHelper
 
-    def initialize info
+    def initialize(info)
       @info = info
       @namespace = @info.namespace
       @classname = @info.safe_name
     end
 
     def build_class
-      unless defined? @klass
-        instantiate_class
-      end
+      instantiate_class unless defined? @klass
       @klass
     end
 
+    def instantiate_class
+      setup_class unless already_set_up
+    end
+
     attr_reader :info
 
     private
@@ -28,7 +30,7 @@ module GirFFI
     end
 
     def lib
-      @lib ||= namespace_module.const_get :Lib
+      @lib ||= namespace_module::Lib
     end
 
     def setup_constants
