@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'gir_ffi/builders/base_type_builder'
 require 'gir_ffi/builders/mapping_method_builder'
 require 'gir_ffi/callback_base'
@@ -25,9 +26,7 @@ module GirFFI
       end
 
       def mapping_method_definition
-        return_value_info = ReturnValueInfo.new info.return_type
-        MappingMethodBuilder.for_callback(info.args,
-                                          return_value_info).method_definition
+        MappingMethodBuilder.for_callback(info).method_definition
       end
 
       def callback_sym
