@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 module GirFFI
   module Builders
     # Builder that generates code to convert closure arguments ('user data')
@@ -9,7 +10,13 @@ module GirFFI
       end
 
       def conversion
-        "GirFFI::InPointer.from_closure_data(#{@callback_argument.call_argument_name}.object_id)"
+        "GirFFI::ArgHelper.store(#{callback_argument_name})"
+      end
+
+      private
+
+      def callback_argument_name
+        @callback_argument.call_argument_name
       end
     end
   end
