@@ -1,4 +1,8 @@
+# frozen_string_literal: true
 require 'gir_ffi/builders/argument_builder'
+require 'gir_ffi/builders/base_argument_builder'
+require 'gir_ffi/builders/null_argument_builder'
+require 'gir_ffi/builders/pointer_value_convertor'
 require 'gir_ffi/variable_name_generator'
 require 'gir_ffi/field_argument_info'
 
@@ -9,6 +13,8 @@ module GirFFI
       # Convertor for fields for field getters. Used when building getter
       # methods.
       class GetterArgumentBuilder < BaseArgumentBuilder
+        attr_reader :array_length_idx
+
         def initialize(var_gen, field_argument_info, field_info,
                        array_length_idx: -1)
           super(var_gen, field_argument_info)
@@ -20,8 +26,7 @@ module GirFFI
         def pre_conversion
           [
             "#{field_ptr} = @struct.to_ptr + #{field_offset}",
-            "#{typed_ptr} = GirFFI::InOutPointer.new(#{field_type_tag}, #{field_ptr})",
-            "#{bare_value} = #{typed_ptr}.to_value"
+            "#{bare_value} = #{pointer_to_value_conversion}"
           ]
         end
 
@@ -49,12 +54,12 @@ module GirFFI
           end
         end
 
-        def array_length_idx
-          @array_length_idx
-        end
-
         private
 
+        def pointer_to_value_conversion
+          PointerValueConvertor.new(field_type_tag).pointer_to_value(field_ptr)
+        end
+
         def field_offset
           @field_info.offset
         end
@@ -72,7 +77,7 @@ module GirFFI
         end
 
         def field_type_tag
-          @field_type_tag ||= @field_info.field_type.tag_or_class.inspect
+          @field_type_tag ||= @field_info.field_type.tag_or_class
         end
 
         def field_type
@@ -113,7 +118,7 @@ module GirFFI
         end
 
         def ownership_transfer
-          :everything
+          :nothing
         end
 
         def caller_allocates?
@@ -126,6 +131,7 @@ module GirFFI
       end
 
       # Builder for field getters
+      # TODO: Derive from BaseMethodBuilder
       class GetterBuilder
         def initialize(info)
           @info = info
@@ -223,8 +229,9 @@ module GirFFI
 
       attr_reader :info
 
-      def initialize(field_info)
+      def initialize(field_info, container_class)
         @info = field_info
+        @container_class = container_class
       end
 
       def build
@@ -233,7 +240,9 @@ module GirFFI
       end
 
       def setup_getter
-        container_class.class_eval getter_def unless container_defines_getter_method?
+        return if container_defines_getter_method?
+        return if hidden_struct_type?
+        container_class.class_eval getter_def
       end
 
       def container_defines_getter_method?
@@ -241,7 +250,9 @@ module GirFFI
       end
 
       def setup_setter
-        container_class.class_eval setter_def if info.writable?
+        return unless info.writable?
+        return if hidden_struct_type?
+        container_class.class_eval setter_def
       end
 
       def getter_def
@@ -254,31 +265,28 @@ module GirFFI
         builder = setter_builder
 
         field_ptr = builder.new_variable
-        typed_ptr = builder.new_variable
 
         <<-CODE.reset_indentation
         def #{info.name}= #{builder.method_argument_name}
           #{field_ptr} = @struct.to_ptr + #{info.offset}
-          #{typed_ptr} = GirFFI::InOutPointer.new(#{field_type_tag}, #{field_ptr})
           #{builder.pre_conversion.join("\n          ")}
-          #{typed_ptr}.set_value #{builder.call_argument_name}
+          #{value_storage(field_ptr, builder)}
         end
         CODE
       end
 
       private
 
+      def value_storage(typed_ptr, builder)
+        PointerValueConvertor.new(field_type_tag).
+          value_to_pointer(typed_ptr, builder.call_argument_name)
+      end
+
       def field_type_tag
-        @field_type_tag ||= info.field_type.tag_or_class.inspect
+        @field_type_tag ||= field_type.tag_or_class
       end
 
-      def container_class
-        @container_class ||= container_module.const_get(container_info.safe_name)
-      end
-
-      def container_module
-        @container_module ||= Object.const_get(container_info.safe_namespace)
-      end
+      attr_reader :container_class
 
       def container_info
         @container_info ||= info.container
@@ -296,6 +304,10 @@ module GirFFI
         @setter_builder ||= ArgumentBuilder.new(VariableNameGenerator.new,
                                                 field_argument_info)
       end
+
+      def hidden_struct_type?
+        field_type.hidden_struct_type?
+      end
     end
   end
 end
