@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'gir_ffi/builders/c_to_ruby_convertor'
 
 module GirFFI
@@ -6,11 +7,10 @@ module GirFFI
     # GValue unpacking. Used by argument builders.
     class FullCToRubyConvertor < CToRubyConvertor
       def conversion
-        base = super
         if @type_info.gvalue?
-          "#{base}.get_value"
+          "GObject::Value.wrap(#{@argument}).get_value"
         else
-          base
+          super
         end
       end
     end
