@@ -1,37 +1,30 @@
+# frozen_string_literal: true
 require 'gir_ffi/variable_name_generator'
 require 'gir_ffi/builders/closure_argument_builder'
-require 'gir_ffi/builders/callback_return_value_builder'
+require 'gir_ffi/builders/closure_return_value_builder'
 require 'gir_ffi/builders/argument_builder_collection'
 require 'gir_ffi/builders/method_template'
 
 module GirFFI
   module Builders
-    # Implements the creation mapping method for a callback or signal
+    # Implements the creation mapping method for a signal
     # handler. This method converts arguments from C to Ruby, and the
     # result from Ruby to C.
-    class MarshallingMethodBuilder
-      def self.for_signal receiver_info, argument_infos, return_value_info
-        new receiver_info, argument_infos, return_value_info
+    class MarshallingMethodBuilder < BaseMethodBuilder
+      def self.for_signal(receiver_info, info)
+        new receiver_info, info
       end
 
-      def initialize receiver_info, argument_infos, return_value_info
-        receiver_builder = make_argument_builder receiver_info
-        argument_builders = argument_infos.map { |arg| make_argument_builder arg }
-        return_value_builder =
-          CallbackReturnValueBuilder.new(variable_generator, return_value_info)
-
-        @argument_builder_collection =
-          ArgumentBuilderCollection.new(return_value_builder, argument_builders,
-                                        receiver_builder: receiver_builder)
-        @template = MethodTemplate.new(self, @argument_builder_collection)
+      def initialize(receiver_info, info)
+        super(info, ClosureReturnValueBuilder,
+              receiver_info: receiver_info,
+              argument_builder_class: ClosureArgumentBuilder)
       end
 
-      def method_definition
-        @template.method_definition
-      end
+      ## Methods used by MethodTemplate
 
       def method_name
-        "marshaller"
+        'marshaller'
       end
 
       def method_arguments
@@ -42,7 +35,7 @@ module GirFFI
         if param_names.size == 1
           ["#{param_names.first} = param_values.first.get_value_plain"]
         else
-          ["#{param_names.join(", ")} = param_values.map(&:get_value_plain)"]
+          ["#{param_names.join(', ')} = param_values.map(&:get_value_plain)"]
         end
       end
 
@@ -71,14 +64,6 @@ module GirFFI
       def param_names
         @param_names ||= @argument_builder_collection.method_argument_names
       end
-
-      def variable_generator
-        @variable_generator ||= VariableNameGenerator.new
-      end
-
-      def make_argument_builder argument_info
-        ClosureArgumentBuilder.new variable_generator, argument_info
-      end
     end
   end
 end
