@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'gir_ffi/builder_helper'
 require 'gir_ffi/module_base'
 require 'gir_ffi/builders/function_builder'
@@ -11,11 +12,10 @@ module GirFFI
     class ModuleBuilder
       include BuilderHelper
 
-      def initialize(namespace, version = nil)
+      def initialize(module_name, namespace: module_name, version: nil)
         @namespace = namespace
         @version = version
-        # FIXME: Pass safe namespace as an argument
-        @safe_namespace = @namespace.gsub(/^./, &:upcase)
+        @safe_namespace = module_name
       end
 
       def generate
@@ -32,6 +32,10 @@ module GirFFI
         true
       end
 
+      def method_available?(method)
+        function_introspection_data(method.to_s) and true
+      end
+
       def build_namespaced_class(classname)
         info = find_namespaced_class_info(classname)
         Builder.build_class info
@@ -62,7 +66,7 @@ module GirFFI
 
       def build_dependencies
         deps = gir.dependencies @namespace
-        deps.each do|dep|
+        deps.each do |dep|
           name, version = dep.split '-'
           Builder.build_module name, version
         end
@@ -109,11 +113,9 @@ module GirFFI
       end
 
       def gir
-        unless defined? @gir
-          @gir = GObjectIntrospection::IRepository.default
-          @gir.require @namespace, @version
+        @gir ||= GObjectIntrospection::IRepository.default.tap do |it|
+          it.require @namespace, @version
         end
-        @gir
       end
     end
   end
