@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 module GirFFI
   # Argument builder that does nothing. Implements the Null Object pattern.
   class NullArgumentBuilder
@@ -32,18 +33,6 @@ module GirFFI
       nil
     end
 
-    def closure_idx
-      -1
-    end
-
-    def destroy_idx
-      -1
-    end
-
-    def block_argument?
-      false
-    end
-
     def post_converted_name
       nil
     end
