@@ -1,7 +1,9 @@
+# frozen_string_literal: true
 require 'gir_ffi/builders/registered_type_builder'
 require 'gir_ffi/builders/with_layout'
 require 'gir_ffi/builders/property_builder'
 require 'gir_ffi/object_base'
+require 'gir_ffi/struct'
 
 module GirFFI
   module Builders
@@ -21,13 +23,11 @@ module GirFFI
       end
 
       def find_signal(signal_name)
-        seek_in_ancestor_infos { |info| info.find_signal signal_name } or
-          raise "Signal #{signal_name} not found"
+        seek_in_ancestor_infos { |info| info.find_signal signal_name }
       end
 
       def find_property(property_name)
-        seek_in_ancestor_infos { |info| info.find_property property_name } or
-          raise "Property #{property_name} not found"
+        seek_in_ancestor_infos { |info| info.find_property property_name }
       end
 
       def object_class_struct
@@ -38,6 +38,21 @@ module GirFFI
         @ancestor_infos ||= [info] + info.interfaces + parent_ancestor_infos
       end
 
+      def eligible_fields
+        info.fields.reject do |finfo|
+          fname = finfo.name
+          fname == 'parent_instance' ||
+            info.find_instance_method("get_#{fname}") ||
+            info.find_property(fname)
+        end
+      end
+
+      def eligible_properties
+        info.properties.reject do |pinfo|
+          info.find_instance_method("get_#{pinfo.name}")
+        end
+      end
+
       protected
 
       def object_class_struct_info
@@ -51,11 +66,8 @@ module GirFFI
         setup_layout
         setup_constants
         stub_methods
-        if info.fundamental?
-          setup_field_accessors
-        else
-          setup_property_accessors
-        end
+        setup_field_accessors
+        setup_property_accessors
         setup_vfunc_invokers
         setup_interfaces
         provide_initializer
@@ -63,16 +75,11 @@ module GirFFI
 
       # FIXME: Private method only used in subclass
       def layout_superclass
-        FFI::Struct
+        GirFFI::Struct
       end
 
       def parent_info
-        unless defined? @parent_info
-          @parent_info = if (parent = info.parent) && parent != info
-                           parent
-                         end
-        end
-        @parent_info
+        info.parent
       end
 
       def superclass
@@ -91,8 +98,14 @@ module GirFFI
         @parent_ancestor_infos ||= parent_builder.ancestor_infos
       end
 
+      def setup_field_accessors
+        eligible_fields.each do |finfo|
+          FieldBuilder.new(finfo, klass).build
+        end
+      end
+
       def setup_property_accessors
-        info.properties.each do |prop|
+        eligible_properties.each do |prop|
           PropertyBuilder.new(prop).build
         end
       end
@@ -101,9 +114,7 @@ module GirFFI
       # TODO: Create object responsible for creating these invokers
       def setup_vfunc_invokers
         info.vfuncs.each do |vfinfo|
-          if (invoker = vfinfo.invoker)
-            define_vfunc_invoker vfinfo.name, invoker.name
-          end
+          define_vfunc_invoker vfinfo.name, vfinfo.invoker_name if vfinfo.has_invoker?
         end
       end
 
@@ -119,11 +130,19 @@ module GirFFI
       def provide_initializer
         return if info.find_method 'new'
 
-        klass.class_eval "
-          def initialize(properties = {})
-            base_initialize(properties)
-          end
-        "
+        if info.abstract?
+          klass.class_eval <<-END
+            def initialize(*)
+              raise NoMethodError
+            end
+          END
+        else
+          klass.class_eval <<-END
+            def initialize(properties = {})
+              base_initialize(properties)
+            end
+          END
+        end
       end
 
       def setup_interfaces
