@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'gir_ffi/builders/base_type_builder'
 require 'gir_ffi/builders/marshalling_method_builder'
 
@@ -11,20 +12,16 @@ module GirFFI
         klass.class_eval marshaller_definition
       end
 
-      def setup_method _method
+      def setup_method(_method)
         nil
       end
 
       def marshaller_definition
-        arg_infos = info.args
-
         container_type_info = ReceiverTypeInfo.new(container_info)
         receiver_info = ReceiverArgumentInfo.new(container_type_info)
-        return_value_info = ReturnValueInfo.new info.return_type
 
         MarshallingMethodBuilder.for_signal(receiver_info,
-                                            arg_infos,
-                                            return_value_info).method_definition
+                                            info).method_definition
       end
 
       def klass
