@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'gir_ffi/builders/callback_builder'
 require 'gir_ffi/builders/constant_builder'
 require 'gir_ffi/builders/enum_builder'
@@ -29,7 +30,7 @@ module GirFFI
         union:                  UnionBuilder,
         unintrospectable_boxed: UnintrospectableBoxedBuilder,
         unintrospectable:       UnintrospectableBuilder
-      }
+      }.freeze
 
       def self.build(info)
         builder_for(info).build_class
