@@ -1,10 +1,14 @@
-require 'gir_ffi/builders/boxed_builder'
+# frozen_string_literal: true
+require 'gir_ffi/builders/registered_type_builder'
+require 'gir_ffi/builders/struct_like'
 
 module GirFFI
   module Builders
     # Implements the creation of a class representing a boxed type for
     # which no data is found in the GIR.
-    class UnintrospectableBoxedBuilder < BoxedBuilder
+    class UnintrospectableBoxedBuilder < RegisteredTypeBuilder
+      include StructLike
+
       def klass
         @klass ||= TypeBuilder::CACHE[target_gtype] ||= Class.new(superclass)
       end
@@ -19,7 +23,7 @@ module GirFFI
       end
 
       def layout_superclass
-        FFI::Struct
+        GirFFI::Struct
       end
     end
   end
