@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'gir_ffi/builders/object_builder'
 require 'gir_ffi/g_type'
 
@@ -10,8 +11,8 @@ module GirFFI
       end
 
       def setup_class
-        register_type
         setup_layout
+        register_type
         setup_constants
         setup_property_accessors
         setup_constructor
@@ -69,22 +70,26 @@ module GirFFI
       end
 
       def class_init_proc
-        proc do |object_class_ptr, _data|
+        proc do |type_class_or_ptr, _data|
+          object_class_ptr = type_class_or_ptr.to_ptr
           setup_properties object_class_ptr
           setup_vfuncs object_class_ptr
         end
       end
 
       def interface_init_proc(interface)
-        proc do |interface_ptr, _data|
+        proc do |interface_or_ptr, _data|
+          interface_ptr = interface_or_ptr.to_ptr
           setup_interface_vfuncs interface, interface_ptr
         end
       end
 
       def instance_size
         size = parent_gtype.instance_size
-        properties.each do
-          size += FFI.type_size(:int32)
+        alignment = struct_class.alignment
+        properties.each do |prop|
+          type_size = FFI.type_size(prop.ffi_type)
+          size += [type_size, alignment].max
         end
         size
       end
@@ -106,13 +111,13 @@ module GirFFI
 
       def property_getter
         proc do |object, _property_id, value, pspec|
-          value.set_value object.send(pspec.get_name)
+          value.set_value object.send(pspec.accessor_name)
         end
       end
 
       def property_setter
         proc do |object, _property_id, value, pspec|
-          object.send("#{pspec.get_name}=", value.get_value)
+          object.send("#{pspec.accessor_name}=", value.get_value)
         end
       end
 
@@ -164,34 +169,92 @@ module GirFFI
       end
 
       def layout_specification
-        parent_spec = [:parent, superclass::Struct, 0]
-        offset = superclass::Struct.size
-        fields_spec = properties.flat_map do |pinfo|
-          spec = [pinfo.name.to_sym, :int32, offset]
-          offset += FFI.type_size(:int32)
+        parent_spec = [:parent, superclass::Struct]
+        offset = parent_gtype.instance_size
+
+        alignment = superclass::Struct.alignment
+        fields_spec = properties.flat_map do |param_info|
+          field_name = param_info.accessor_name
+          ffi_type = param_info.ffi_type
+          type_size = FFI.type_size(ffi_type)
+          spec = [field_name, ffi_type, offset]
+          offset += [type_size, alignment].max
           spec
         end
         parent_spec + fields_spec
       end
 
-      def setup_property_accessors
-        properties.each do |pinfo|
-          setup_accessors_for_param_info pinfo
+      # TODO: Move this to its own file.
+      # TODO: See if this or FieldTypeInfo can be merged with with
+      # UserDefinedPropertyInfo.
+      class UserDefinedPropertyFieldInfo
+        # Field info for user-defined property
+        class FieldTypeInfo
+          include InfoExt::ITypeInfo
+
+          def initialize(property_info)
+            @property_info = property_info
+          end
+
+          def tag
+            @property_info.type_tag
+          end
+
+          def pointer?
+            @property_info.pointer_type?
+          end
+
+          def interface_type
+            @property_info.interface_type_tag if tag == :interface
+          end
+
+          def hidden_struct_type?
+            false
+          end
+
+          def interface_class
+            Builder.build_by_gtype @property_info.value_type if tag == :interface
+          end
+
+          def interface_class_name
+            interface_class.name if tag == :interface
+          end
+        end
+
+        def initialize(property_info, container, offset)
+          @property_info = property_info
+          @container = container
+          @offset = offset
+        end
+
+        attr_reader :container, :offset
+
+        def name
+          @property_info.accessor_name
+        end
+
+        def field_type
+          @field_type ||= FieldTypeInfo.new @property_info
+        end
+
+        def related_array_length_field
+          nil
+        end
+
+        def writable?
+          @property_info.writable?
         end
       end
 
-      def setup_accessors_for_param_info(pinfo)
-        field_name = pinfo.name
-        code = <<-CODE
-        def #{field_name}
-          @struct[:#{field_name}]
+      def setup_property_accessors
+        offset = parent_gtype.instance_size
+        alignment = struct_class.alignment
+        properties.each do |param_info|
+          field_info = UserDefinedPropertyFieldInfo.new(param_info, info, offset)
+          type_size = FFI.type_size(param_info.ffi_type)
+          offset += [type_size, alignment].max
+          FieldBuilder.new(field_info, klass).build
         end
-        def #{field_name}= val
-          @struct[:#{field_name}] = val
-        end
-        CODE
-
-        klass.class_eval code
       end
 
       def method_introspection_data(_method)
