@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'gir_ffi/type_base'
 
 module GirFFI
@@ -12,6 +13,7 @@ module GirFFI
       FFI::Type::POINTER
     end
 
+    # TODO: Return instance of this class
     def self.from_native(value, _context)
       return nil if !value || value.null?
       FFI::Function.new(gir_ffi_builder.return_ffi_type,
@@ -37,8 +39,8 @@ module GirFFI
       CALLBACKS[prc.object_id] = prc
     end
 
-    def self.drop_callback(key)
-      CALLBACKS.delete key
+    def self.drop_callback(prc)
+      CALLBACKS.delete prc.object_id
     end
 
     # Create Callback from a Proc. Makes sure arguments are properly wrapped,
@@ -73,6 +75,10 @@ module GirFFI
                      end
     end
 
+    def to_ptr
+      to_native.to_ptr
+    end
+
     def self.copy_value_to_pointer(value, pointer)
       pointer.put_pointer 0, to_native(value, nil)
     end
