@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 module GirFFI
   # Class to represent argument info for the argument of a setter method.
   # Implements the necessary parts of IArgumentInfo's interface.
@@ -5,7 +6,7 @@ module GirFFI
   class FieldArgumentInfo
     attr_reader :name, :argument_type
 
-    def initialize name, type
+    def initialize(name, type)
       @name = name
       @argument_type = type
     end
@@ -14,6 +15,10 @@ module GirFFI
       :in
     end
 
+    def ownership_transfer
+      :everything
+    end
+
     def skip?
       false
     end
