@@ -1,17 +1,18 @@
+# frozen_string_literal: true
 module GirFFI
   module InfoExt
     # Extensions for GObjectIntrospection::ICallbackInfo needed by GirFFI
     module ICallbackInfo
-      def to_ffitype
+      def to_ffi_type
         Builder.build_class(self)
       end
 
       def argument_ffi_types
-        args.map(&:to_callback_ffitype)
+        args.map(&:to_callback_ffi_type)
       end
 
       def return_ffi_type
-        return_type.to_callback_ffitype
+        return_type.to_callback_ffi_type
       end
     end
   end
