@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 module GirFFI
   module InfoExt
     # Extensions for GObjectIntrospection::IFunctionInfo needed by GirFFI
@@ -10,7 +11,7 @@ module GirFFI
       end
 
       def return_ffi_type
-        return_type.to_ffitype
+        return_type.to_ffi_type
       end
     end
   end
