@@ -1,9 +1,10 @@
+# frozen_string_literal: true
 module GirFFI
   module InfoExt
     # Extensions for GObjectIntrospection::IPropertyInfo needed by GirFFI
     module IPropertyInfo
       def getter_name
-        name.gsub(/-/, '_')
+        name.tr('-', '_')
       end
 
       def setter_name
