@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 module GirFFI
   module InfoExt
     # Extensions for GObjectIntrospection::ISignalInfo needed by GirFFI
@@ -9,18 +10,17 @@ module GirFFI
       #
       # @return [GObject::RubyClosure]  The signal handler closure, ready to be
       #                                 passed as a GClosure to C.
-      def wrap_in_closure &block
+      def wrap_in_closure(&block)
         bldr = Builders::SignalClosureBuilder.new(self)
         bldr.build_class.new(&block)
       end
 
-      # TODO: Use argument info to convert out arguments and array lengths.
-      def arguments_to_gvalues instance, arguments
-        arg_values = args.zip(arguments).map do |info, arg|
-          info.argument_type.make_g_value.set_value(arg)
+      def arguments_to_gvalues(instance, arguments)
+        arg_g_values = args.zip(arguments).map do |info, arg|
+          info.argument_type.make_g_value.tap { |it| it.set_value arg }
         end
 
-        arg_values.unshift GObject::Value.wrap_instance(instance)
+        arg_g_values.unshift GObject::Value.wrap_instance(instance)
       end
 
       def gvalue_for_return_value
