@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 module GirFFI
   module InfoExt
     # Extension module provinding a #safe_name method suitable for types.
@@ -5,8 +6,8 @@ module GirFFI
       def safe_name
         name.tr('-', '_').gsub(/^./) do |char|
           case char
-          when "_"
-            "Private___"
+          when '_'
+            'Private___'
           else
             char.upcase
           end
