@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'gir_ffi/type_base'
 
 module GirFFI
@@ -6,7 +7,7 @@ module GirFFI
   module RegisteredTypeBase
     include TypeBase
 
-    def get_gtype
+    def gtype
       self::G_TYPE
     end
   end
