@@ -1,10 +1,13 @@
+# frozen_string_literal: true
 module GirFFI
   # Represents a return value with the same interface as IArgumentInfo
   class ReturnValueInfo
     attr_reader :argument_type
+    attr_reader :ownership_transfer
 
-    def initialize type, skip = false
+    def initialize(type, ownership_transfer, skip)
       @argument_type = type
+      @ownership_transfer = ownership_transfer
       @skip = skip
     end
 
