@@ -1,12 +1,13 @@
+# frozen_string_literal: true
 module GirFFI
   # Base module for all generated GLib types.
   module TypeBase
     def gir_ffi_builder
-      const_get :GIR_FFI_BUILDER
+      self::GIR_FFI_BUILDER
     end
 
     def gir_info
-      const_get :GIR_INFO
+      self::GIR_INFO
     end
   end
 end
