@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+require 'gir_ffi/boolean'
 require 'gir_ffi/sized_array'
 
 module GirFFI
@@ -13,40 +15,42 @@ module GirFFI
       glist:     :pointer,
       gslist:    :pointer,
       strv:      :pointer,
-      object:    :pointer,
-      struct:    :pointer,
+      interface: :pointer,
       error:     :pointer,
       ptr_array: :pointer,
       array:     :pointer,
       c:         GirFFI::SizedArray,
       utf8:      :pointer,
       GType:     gsize_type,
-      gboolean:  GLib::Boolean,
+      gboolean:  GirFFI::Boolean,
       gunichar:  :uint32,
       gint8:     :int8,
       guint8:    :uint8,
       gint16:    :int16,
       guint16:   :uint16,
       gint:      :int,
+      guint:     :uint,
       gint32:    :int32,
       guint32:   :uint32,
       gint64:    :int64,
       guint64:   :uint64,
+      glong:     :long,
+      gulong:    :ulong,
       gsize:     gsize_type,
       gfloat:    :float,
       gdouble:   :double,
       void:      :void
-    }
+    }.freeze
 
-    def self.map_basic_type type
+    def self.map_basic_type(type)
       sym = type.to_sym
       TAG_TYPE_MAP[sym] || sym
     end
 
-    def self.type_specification_to_ffitype type
+    def self.type_specification_to_ffi_type(type)
       case type
       when Module
-        type.to_ffitype
+        type.to_ffi_type
       when Array
         type[0]
       else
