@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 module GirFFI
   # Represents a null-terminated array.
   class ZeroTerminated
@@ -5,7 +6,7 @@ module GirFFI
 
     attr_reader :element_type
 
-    def initialize elm_t, ptr
+    def initialize(elm_t, ptr)
       @element_type = elm_t
       @ptr = ptr
     end
@@ -14,11 +15,11 @@ module GirFFI
       @ptr
     end
 
-    def self.from type, arg
+    def self.from(type, arg)
       new type, InPointer.from_array(type, arg)
     end
 
-    def self.wrap type, arg
+    def self.wrap(type, arg)
       new type, arg
     end
 
@@ -31,14 +32,14 @@ module GirFFI
       end
     end
 
-    def == other
+    def ==(other)
       to_a == other.to_a
     end
 
     private
 
-    def read_value offset
-      val = @ptr.send(getter_method, offset)
+    def read_value(offset)
+      val = fetch_value(offset)
       val unless val.zero?
     end
 
@@ -46,7 +47,18 @@ module GirFFI
       @getter_method ||= "get_#{ffi_type}"
     end
 
-    def wrap_value val
+    def fetch_value(offset)
+      case ffi_type
+      when Module
+        ffi_type.get_value_from_pointer(@ptr, offset)
+      when Symbol
+        @ptr.send(getter_method, offset)
+      else
+        raise NotImplementedError
+      end
+    end
+
+    def wrap_value(val)
       case element_type
       when Array
         element_type.last.wrap val
@@ -58,7 +70,7 @@ module GirFFI
     end
 
     def ffi_type
-      @ffi_type ||= TypeMap.type_specification_to_ffitype element_type
+      @ffi_type ||= TypeMap.type_specification_to_ffi_type element_type
     end
 
     def ffi_type_size
