@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'gir_ffi_test_helper'
 
 GirFFI.setup :Regress
@@ -255,7 +256,7 @@ describe GObject::Value do
       gv.get_value.must_equal value
     end
 
-    it 'unwraps a ulong' do
+    it 'unwraps an ulong' do
       value = FFI.type_size(:long) == 8 ? 0x1234_5678_9012_3456 : 0x1234_5678
       gv = GObject::Value.for_gtype GObject::TYPE_ULONG
       gv.set_ulong value
@@ -290,6 +291,17 @@ describe GObject::Value do
       result.must_be_kind_of GLib::Array
       result.reset_typespec(:uint).to_a.must_equal [1, 2, 3]
     end
+
+    it 'unwraps a Strv' do
+      strv = GLib::Strv.from %w(foo bar)
+      val = GObject::Value.for_gtype GObject::TYPE_STRV
+      val.set_boxed strv
+
+      result = val.get_value
+
+      result.must_be_kind_of GLib::Strv
+      result.to_a.must_equal %w(foo bar)
+    end
   end
 
   describe '::from' do
@@ -333,12 +345,8 @@ describe GObject::Value do
   end
 
   describe 'upon garbage collection' do
-    before do
-      GirFFI.setup :GIMarshallingTests
-    end
-
     it 'restores the underlying GValue to its pristine state' do
-      if defined?(RUBY_ENGINE) && %w(jruby rbx).include?(RUBY_ENGINE)
+      if jruby? || rubinius?
         skip 'cannot be reliably tested on JRuby and Rubinius'
       end
 
@@ -349,6 +357,8 @@ describe GObject::Value do
       value.current_gtype_name.must_equal 'gint'
 
       GC.start
+      # Creating a new object is sometimes needed to trigger enough garbage collection.
+      GObject::Value.new
       sleep 1
       GC.start
       GC.start
