@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'gir_ffi_test_helper'
 
 describe GirFFI::Builders::ArgumentBuilder do
@@ -5,6 +6,23 @@ describe GirFFI::Builders::ArgumentBuilder do
   let(:builder) { GirFFI::Builders::ArgumentBuilder.new(var_gen, arg_info) }
 
   describe 'for an argument with direction :in' do
+    describe 'for :c' do
+      describe 'with full ownership transfer' do
+        let(:arg_info) do
+          get_introspection_data('GIMarshallingTests', 'array_struct_take_in').args[0]
+        end
+
+        it 'has the correct value for #pre_conversion' do
+          builder.pre_conversion.
+            must_equal ['_v1 = GirFFI::SizedArray.copy_from([:pointer, GIMarshallingTests::BoxedStruct], -1, structs)']
+        end
+
+        it 'has the correct value for #post_conversion' do
+          builder.post_conversion.must_equal []
+        end
+      end
+    end
+
     describe 'for :callback' do
       let(:arg_info) do
         get_introspection_data('Regress', 'test_callback_destroy_notify').args[0]
@@ -47,7 +65,7 @@ describe GirFFI::Builders::ArgumentBuilder do
         end
 
         it 'has the correct value for #pre_conversion' do
-          builder.pre_conversion.must_equal ['_v1 = GirFFI::InPointer.from(:void, user_data)']
+          builder.pre_conversion.must_equal ['_v1 = user_data']
         end
 
         it 'has the correct value for #post_conversion' do
@@ -67,7 +85,7 @@ describe GirFFI::Builders::ArgumentBuilder do
         end
 
         it 'has the correct value for #pre_conversion' do
-          builder.pre_conversion.must_equal ['_v1 = GirFFI::InPointer.from_closure_data(foo.object_id)']
+          builder.pre_conversion.must_equal ['_v1 = GirFFI::ArgHelper.store(foo)']
         end
 
         it 'has the correct value for #post_conversion' do
@@ -82,11 +100,11 @@ describe GirFFI::Builders::ArgumentBuilder do
       let(:arg_info) { get_introspection_data('GIMarshallingTests', 'genum_out').args[0] }
 
       it 'has the correct value for #pre_conversion' do
-        builder.pre_conversion.must_equal ['_v1 = GirFFI::InOutPointer.for GIMarshallingTests::GEnum']
+        builder.pre_conversion.must_equal ['_v1 = FFI::MemoryPointer.new GIMarshallingTests::GEnum']
       end
 
       it 'has the correct value for #post_conversion' do
-        builder.post_conversion.must_equal ['_v2 = _v1.to_value']
+        builder.post_conversion.must_equal ['_v2 = GIMarshallingTests::GEnum.get_value_from_pointer(_v1, 0)']
       end
     end
 
@@ -96,25 +114,42 @@ describe GirFFI::Builders::ArgumentBuilder do
       end
 
       it 'has the correct value for #pre_conversion' do
-        builder.pre_conversion.must_equal ['_v1 = GirFFI::InOutPointer.for GIMarshallingTests::Flags']
+        builder.pre_conversion.must_equal ['_v1 = FFI::MemoryPointer.new GIMarshallingTests::Flags']
       end
 
       it 'has the correct value for #post_conversion' do
-        builder.post_conversion.must_equal ['_v2 = _v1.to_value']
+        builder.post_conversion.must_equal ['_v2 = GIMarshallingTests::Flags.get_value_from_pointer(_v1, 0)']
       end
     end
 
     describe 'for :object' do
-      let(:arg_info) do
-        get_method_introspection_data('Regress', 'TestObj', 'null_out').args[0]
+      describe 'with full ownership transfer' do
+        let(:arg_info) do
+          get_method_introspection_data('Regress', 'TestObj', 'null_out').args[0]
+        end
+
+        it 'has the correct value for #pre_conversion' do
+          builder.pre_conversion.must_equal ['_v1 = FFI::MemoryPointer.new :pointer']
+        end
+
+        it 'has the correct value for #post_conversion' do
+          builder.post_conversion.must_equal ['_v2 = Regress::TestObj.wrap(_v1.get_pointer(0))']
+        end
       end
 
-      it 'has the correct value for #pre_conversion' do
-        builder.pre_conversion.must_equal ['_v1 = GirFFI::InOutPointer.for [:pointer, Regress::TestObj]']
-      end
+      describe 'with no ownership transfer' do
+        let(:arg_info) do
+          get_method_introspection_data('GIMarshallingTests', 'Object', 'none_out').args[0]
+        end
 
-      it 'has the correct value for #post_conversion' do
-        builder.post_conversion.must_equal ['_v2 = Regress::TestObj.wrap(_v1.to_value)']
+        it 'has the correct value for #pre_conversion' do
+          builder.pre_conversion.must_equal ['_v1 = FFI::MemoryPointer.new :pointer']
+        end
+
+        it 'has the correct value for #post_conversion' do
+          builder.post_conversion.
+            must_equal ['_v2 = GIMarshallingTests::Object.wrap(_v1.get_pointer(0)).tap { |it| it && it.ref }']
+        end
       end
     end
 
@@ -126,12 +161,13 @@ describe GirFFI::Builders::ArgumentBuilder do
 
         it 'has the correct value for #pre_conversion' do
           builder.pre_conversion.must_equal [
-            '_v1 = GirFFI::InOutPointer.for [:pointer, GIMarshallingTests::BoxedStruct]'
+            '_v1 = FFI::MemoryPointer.new :pointer'
           ]
         end
 
         it 'has the correct value for #post_conversion' do
-          builder.post_conversion.must_equal ['_v2 = GIMarshallingTests::BoxedStruct.wrap(_v1.to_value)']
+          builder.post_conversion.
+            must_equal ['_v2 = GIMarshallingTests::BoxedStruct.wrap_copy(_v1.get_pointer(0))']
         end
       end
 
@@ -156,11 +192,11 @@ describe GirFFI::Builders::ArgumentBuilder do
       end
 
       it 'has the correct value for #pre_conversion' do
-        builder.pre_conversion.must_equal ['_v1 = GirFFI::InOutPointer.for [:pointer, :strv]']
+        builder.pre_conversion.must_equal ['_v1 = FFI::MemoryPointer.new :pointer']
       end
 
       it 'has the correct value for #post_conversion' do
-        builder.post_conversion.must_equal ['_v2 = GLib::Strv.wrap(_v1.to_value)']
+        builder.post_conversion.must_equal ['_v2 = GLib::Strv.wrap(_v1.get_pointer(0))']
       end
     end
 
@@ -171,11 +207,11 @@ describe GirFFI::Builders::ArgumentBuilder do
         end
 
         it 'has the correct value for #pre_conversion' do
-          builder.pre_conversion.must_equal ['_v1 = GirFFI::InOutPointer.for [:pointer, :array]']
+          builder.pre_conversion.must_equal ['_v1 = FFI::MemoryPointer.new :pointer']
         end
 
         it 'has the correct value for #post_conversion' do
-          builder.post_conversion.must_equal ['_v2 = GLib::Array.wrap(:utf8, _v1.to_value)']
+          builder.post_conversion.must_equal ['_v2 = GLib::Array.wrap(:utf8, _v1.get_pointer(0))']
         end
       end
 
@@ -206,11 +242,11 @@ describe GirFFI::Builders::ArgumentBuilder do
       end
 
       it 'has the correct value for #pre_conversion' do
-        builder.pre_conversion.must_equal ['_v1 = GirFFI::InOutPointer.for [:pointer, :ptr_array]']
+        builder.pre_conversion.must_equal ['_v1 = FFI::MemoryPointer.new :pointer']
       end
 
       it 'has the correct value for #post_conversion' do
-        builder.post_conversion.must_equal ['_v2 = GLib::PtrArray.wrap(:utf8, _v1.to_value)']
+        builder.post_conversion.must_equal ['_v2 = GLib::PtrArray.wrap(:utf8, _v1.get_pointer(0))']
       end
     end
 
@@ -220,11 +256,11 @@ describe GirFFI::Builders::ArgumentBuilder do
       end
 
       it 'has the correct value for #pre_conversion' do
-        builder.pre_conversion.must_equal ['_v1 = GirFFI::InOutPointer.for [:pointer, :error]']
+        builder.pre_conversion.must_equal ['_v1 = FFI::MemoryPointer.new :pointer']
       end
 
       it 'has the correct value for #post_conversion' do
-        builder.post_conversion.must_equal ['_v2 = GLib::Error.wrap(_v1.to_value)']
+        builder.post_conversion.must_equal ['_v2 = GLib::Error.wrap(_v1.get_pointer(0))']
       end
     end
 
@@ -235,11 +271,11 @@ describe GirFFI::Builders::ArgumentBuilder do
         end
 
         it 'has the correct value for #pre_conversion' do
-          builder.pre_conversion.must_equal ['_v1 = GirFFI::InOutPointer.for [:pointer, :c]']
+          builder.pre_conversion.must_equal ['_v1 = FFI::MemoryPointer.new :pointer']
         end
 
         it 'has the correct value for #post_conversion' do
-          builder.post_conversion.must_equal ['_v2 = GirFFI::SizedArray.wrap(:gint32, 4, _v1.to_value)']
+          builder.post_conversion.must_equal ['_v2 = GirFFI::SizedArray.wrap(:gint32, 4, _v1.get_pointer(0))']
         end
       end
 
@@ -255,11 +291,11 @@ describe GirFFI::Builders::ArgumentBuilder do
         end
 
         it 'has the correct value for #pre_conversion' do
-          builder.pre_conversion.must_equal ['_v1 = GirFFI::InOutPointer.for [:pointer, :c]']
+          builder.pre_conversion.must_equal ['_v1 = FFI::MemoryPointer.new :pointer']
         end
 
         it 'has the correct value for #post_conversion' do
-          builder.post_conversion.must_equal ['_v2 = GirFFI::SizedArray.wrap(:gint32, bar, _v1.to_value)']
+          builder.post_conversion.must_equal ['_v2 = GirFFI::SizedArray.wrap(:gint32, bar, _v1.get_pointer(0))']
         end
       end
     end
@@ -270,11 +306,11 @@ describe GirFFI::Builders::ArgumentBuilder do
       end
 
       it 'has the correct value for #pre_conversion' do
-        builder.pre_conversion.must_equal ['_v1 = GirFFI::InOutPointer.for [:pointer, :glist]']
+        builder.pre_conversion.must_equal ['_v1 = FFI::MemoryPointer.new :pointer']
       end
 
       it 'has the correct value for #post_conversion' do
-        builder.post_conversion.must_equal ['_v2 = GLib::List.wrap(:utf8, _v1.to_value)']
+        builder.post_conversion.must_equal ['_v2 = GLib::List.wrap(:utf8, _v1.get_pointer(0))']
       end
     end
 
@@ -284,11 +320,11 @@ describe GirFFI::Builders::ArgumentBuilder do
       end
 
       it 'has the correct value for #pre_conversion' do
-        builder.pre_conversion.must_equal ['_v1 = GirFFI::InOutPointer.for [:pointer, :gslist]']
+        builder.pre_conversion.must_equal ['_v1 = FFI::MemoryPointer.new :pointer']
       end
 
       it 'has the correct value for #post_conversion' do
-        builder.post_conversion.must_equal ['_v2 = GLib::SList.wrap(:utf8, _v1.to_value)']
+        builder.post_conversion.must_equal ['_v2 = GLib::SList.wrap(:utf8, _v1.get_pointer(0))']
       end
     end
 
@@ -298,11 +334,67 @@ describe GirFFI::Builders::ArgumentBuilder do
       end
 
       it 'has the correct value for #pre_conversion' do
-        builder.pre_conversion.must_equal ['_v1 = GirFFI::InOutPointer.for [:pointer, [:ghash, :utf8, :utf8]]']
+        builder.pre_conversion.must_equal ['_v1 = FFI::MemoryPointer.new :pointer']
       end
 
       it 'has the correct value for #post_conversion' do
-        builder.post_conversion.must_equal ['_v2 = GLib::HashTable.wrap([:utf8, :utf8], _v1.to_value)']
+        builder.post_conversion.must_equal ['_v2 = GLib::HashTable.wrap([:utf8, :utf8], _v1.get_pointer(0))']
+      end
+    end
+
+    describe 'for :utf8' do
+      describe 'with full ownership transfer' do
+        let(:arg_info) do
+          get_introspection_data('GIMarshallingTests', 'utf8_full_out').args[0]
+        end
+
+        it 'has the correct value for #pre_conversion' do
+          builder.pre_conversion.must_equal ['_v1 = FFI::MemoryPointer.new :pointer']
+        end
+
+        it 'has the correct value for #post_conversion' do
+          builder.post_conversion.must_equal [
+            '_v2 = GirFFI::AllocationHelper.free_after _v1.get_pointer(0), &:to_utf8'
+          ]
+        end
+
+        it 'has the correct value for #return_value_name' do
+          builder.post_conversion
+          builder.return_value_name.must_equal '_v2'
+        end
+      end
+
+      describe 'with no ownership transfer' do
+        let(:arg_info) do
+          get_introspection_data('GIMarshallingTests', 'utf8_none_out').args[0]
+        end
+
+        it 'has the correct value for #pre_conversion' do
+          builder.pre_conversion.must_equal ['_v1 = FFI::MemoryPointer.new :pointer']
+        end
+
+        it 'has the correct value for #post_conversion' do
+          builder.post_conversion.must_equal ['_v2 = _v1.get_pointer(0).to_utf8']
+        end
+      end
+    end
+
+    describe 'for :void' do
+      let(:function_info) do
+        get_introspection_data('Everything', 'one_outparam_gpointer')
+      end
+      let(:arg_info) { function_info.args[0] }
+
+      before do
+        skip unless function_info
+      end
+
+      it 'has the correct value for #pre_conversion' do
+        builder.pre_conversion.must_equal ['_v1 = FFI::MemoryPointer.new :pointer']
+      end
+
+      it 'has the correct value for #post_conversion' do
+        builder.post_conversion.must_equal ['_v2 = _v1.get_pointer(0)']
       end
     end
   end
@@ -314,12 +406,12 @@ describe GirFFI::Builders::ArgumentBuilder do
       end
 
       it 'has the correct value for #pre_conversion' do
-        builder.pre_conversion.must_equal ['_v1 = GirFFI::InOutPointer.for GIMarshallingTests::Enum',
-                                           "_v1.set_value #{arg_info.name}"]
+        builder.pre_conversion.must_equal ['_v1 = FFI::MemoryPointer.new GIMarshallingTests::Enum',
+                                           "GIMarshallingTests::Enum.copy_value_to_pointer(#{arg_info.name}, _v1)"]
       end
 
       it 'has the correct value for #post_conversion' do
-        builder.post_conversion.must_equal ['_v2 = _v1.to_value']
+        builder.post_conversion.must_equal ['_v2 = GIMarshallingTests::Enum.get_value_from_pointer(_v1, 0)']
       end
     end
 
@@ -329,12 +421,13 @@ describe GirFFI::Builders::ArgumentBuilder do
       end
 
       it 'has the correct value for #pre_conversion' do
-        builder.pre_conversion.must_equal ['_v1 = GirFFI::InOutPointer.for GIMarshallingTests::NoTypeFlags',
-                                           "_v1.set_value #{arg_info.name}"]
+        builder.pre_conversion.
+          must_equal ['_v1 = FFI::MemoryPointer.new GIMarshallingTests::NoTypeFlags',
+                      "GIMarshallingTests::NoTypeFlags.copy_value_to_pointer(#{arg_info.name}, _v1)"]
       end
 
       it 'has the correct value for #post_conversion' do
-        builder.post_conversion.must_equal ['_v2 = _v1.to_value']
+        builder.post_conversion.must_equal ['_v2 = GIMarshallingTests::NoTypeFlags.get_value_from_pointer(_v1, 0)']
       end
     end
 
@@ -344,16 +437,16 @@ describe GirFFI::Builders::ArgumentBuilder do
       end
 
       it 'has the correct value for method_argument_name' do
-        builder.method_argument_name.must_equal "#{arg_info.name}"
+        builder.method_argument_name.must_equal arg_info.name
       end
 
       it 'has the correct value for #pre_conversion' do
-        builder.pre_conversion.must_equal ['_v1 = GirFFI::InOutPointer.for :gint32',
-                                           "_v1.set_value #{arg_info.name}"]
+        builder.pre_conversion.must_equal ['_v1 = FFI::MemoryPointer.new :int32',
+                                           "_v1.put_int32 0, #{arg_info.name}"]
       end
 
       it 'has the correct value for #post_conversion' do
-        builder.post_conversion.must_equal ['_v2 = _v1.to_value']
+        builder.post_conversion.must_equal ['_v2 = _v1.get_int32(0)']
       end
     end
 
@@ -373,12 +466,12 @@ describe GirFFI::Builders::ArgumentBuilder do
 
       it 'has the correct value for #pre_conversion' do
         builder.pre_conversion.must_equal ['n_ints = ints.nil? ? 0 : ints.length',
-                                           '_v1 = GirFFI::InOutPointer.for :gint32',
-                                           '_v1.set_value n_ints']
+                                           '_v1 = FFI::MemoryPointer.new :int32',
+                                           '_v1.put_int32 0, n_ints']
       end
 
       it 'has the correct value for #post_conversion' do
-        builder.post_conversion.must_equal ['_v2 = _v1.to_value']
+        builder.post_conversion.must_equal ['_v2 = _v1.get_int32(0)']
       end
     end
 
@@ -388,12 +481,12 @@ describe GirFFI::Builders::ArgumentBuilder do
       end
 
       it 'has the correct value for #pre_conversion' do
-        builder.pre_conversion.must_equal ['_v1 = GirFFI::InOutPointer.for [:pointer, :strv]',
-                                           '_v1.set_value GLib::Strv.from(g_strv)']
+        builder.pre_conversion.must_equal ['_v1 = FFI::MemoryPointer.new :pointer',
+                                           '_v1.put_pointer 0, GLib::Strv.from(g_strv)']
       end
 
       it 'has the correct value for #post_conversion' do
-        builder.post_conversion.must_equal ['_v2 = GLib::Strv.wrap(_v1.to_value)']
+        builder.post_conversion.must_equal ['_v2 = GLib::Strv.wrap(_v1.get_pointer(0))']
       end
     end
 
@@ -403,12 +496,12 @@ describe GirFFI::Builders::ArgumentBuilder do
       end
 
       it 'has the correct value for #pre_conversion' do
-        builder.pre_conversion.must_equal ['_v1 = GirFFI::InOutPointer.for [:pointer, :ptr_array]',
-                                           '_v1.set_value GLib::PtrArray.from(:utf8, parray_)']
+        builder.pre_conversion.must_equal ['_v1 = FFI::MemoryPointer.new :pointer',
+                                           '_v1.put_pointer 0, GLib::PtrArray.from(:utf8, parray_)']
       end
 
       it 'has the correct value for #post_conversion' do
-        builder.post_conversion.must_equal ['_v2 = GLib::PtrArray.wrap(:utf8, _v1.to_value)']
+        builder.post_conversion.must_equal ['_v2 = GLib::PtrArray.wrap(:utf8, _v1.get_pointer(0))']
       end
     end
 
@@ -418,12 +511,32 @@ describe GirFFI::Builders::ArgumentBuilder do
       end
 
       it 'has the correct value for #pre_conversion' do
-        builder.pre_conversion.must_equal ['_v1 = GirFFI::InOutPointer.for :utf8',
-                                           '_v1.set_value GirFFI::InPointer.from(:utf8, utf8)']
+        builder.pre_conversion.must_equal ['_v1 = FFI::MemoryPointer.new :pointer',
+                                           '_v1.put_pointer 0, GirFFI::InPointer.from_utf8(utf8)']
       end
 
       it 'has the correct value for #post_conversion' do
-        builder.post_conversion.must_equal ['_v2 = _v1.to_value.to_utf8']
+        builder.post_conversion.must_equal ['_v2 = _v1.get_pointer(0).to_utf8']
+      end
+    end
+
+    describe 'for :struct' do
+      describe 'with full ownership transfer' do
+        let(:arg_info) do
+          get_introspection_data('GIMarshallingTests', 'boxed_struct_inout').args[0]
+        end
+
+        it 'has the correct value for #pre_conversion' do
+          builder.pre_conversion.must_equal [
+            '_v1 = FFI::MemoryPointer.new :pointer',
+            '_v1.put_pointer 0, GIMarshallingTests::BoxedStruct.copy_from(struct_)'
+          ]
+        end
+
+        it 'has the correct value for #post_conversion' do
+          builder.post_conversion.
+            must_equal ['_v2 = GIMarshallingTests::BoxedStruct.wrap_own(_v1.get_pointer(0))']
+        end
       end
     end
 
@@ -435,14 +548,14 @@ describe GirFFI::Builders::ArgumentBuilder do
 
         it 'has the correct value for #pre_conversion' do
           builder.pre_conversion.must_equal [
-            "GirFFI::ArgHelper.check_fixed_array_size 4, ints, \"ints\"",
-            '_v1 = GirFFI::InOutPointer.for [:pointer, :c]',
-            '_v1.set_value GirFFI::SizedArray.from(:gint32, 4, ints)'
+            'GirFFI::ArgHelper.check_fixed_array_size 4, ints, "ints"',
+            '_v1 = FFI::MemoryPointer.new :pointer',
+            '_v1.put_pointer 0, GirFFI::SizedArray.from(:gint32, 4, ints)'
           ]
         end
 
         it 'has the correct value for #post_conversion' do
-          builder.post_conversion.must_equal ['_v2 = GirFFI::SizedArray.wrap(:gint32, 4, _v1.to_value)']
+          builder.post_conversion.must_equal ['_v2 = GirFFI::SizedArray.wrap(:gint32, 4, _v1.get_pointer(0))']
         end
       end
 
@@ -462,13 +575,13 @@ describe GirFFI::Builders::ArgumentBuilder do
 
         it 'has the correct value for #pre_conversion' do
           builder.pre_conversion.must_equal [
-            '_v1 = GirFFI::InOutPointer.for [:pointer, :c]',
-            '_v1.set_value GirFFI::SizedArray.from(:gint32, -1, ints)'
+            '_v1 = FFI::MemoryPointer.new :pointer',
+            '_v1.put_pointer 0, GirFFI::SizedArray.copy_from(:gint32, -1, ints)'
           ]
         end
 
         it 'has the correct value for #post_conversion' do
-          builder.post_conversion.must_equal ['_v3 = GirFFI::SizedArray.wrap(:gint32, _v2, _v1.to_value)']
+          builder.post_conversion.must_equal ['_v3 = GirFFI::SizedArray.wrap(:gint32, _v2, _v1.get_pointer(0))']
         end
       end
     end
@@ -502,8 +615,8 @@ describe GirFFI::Builders::ArgumentBuilder do
     end
 
     it 'has the correct value for #pre_conversion' do
-      builder.pre_conversion.must_equal ['_v1 = GirFFI::InOutPointer.for :gint32',
-                                         '_v1.set_value 0']
+      builder.pre_conversion.must_equal ['_v1 = FFI::MemoryPointer.new :int32',
+                                         '_v1.put_int32 0, 0']
     end
 
     it 'has the correct value for #post_conversion' do
@@ -521,11 +634,27 @@ describe GirFFI::Builders::ArgumentBuilder do
     end
 
     it 'has the correct value for #pre_conversion' do
-      builder.pre_conversion.must_equal ['_v1 = GirFFI::InOutPointer.for :gint32']
+      builder.pre_conversion.must_equal ['_v1 = FFI::MemoryPointer.new :int32']
     end
 
     it 'has the correct value for #post_conversion' do
       builder.post_conversion.must_equal []
     end
   end
+
+  describe 'for an argument with direction :error' do
+    let(:arg_info) { GirFFI::ErrorArgumentInfo.new }
+
+    it 'has the correct value for #pre_conversion' do
+      builder.pre_conversion.must_equal [
+        '_v1 = FFI::MemoryPointer.new(:pointer).write_pointer nil'
+      ]
+    end
+
+    it 'has the correct value for #post_conversion' do
+      builder.post_conversion.must_equal [
+        'GirFFI::ArgHelper.check_error(_v1)'
+      ]
+    end
+  end
 end
