@@ -1,7 +1,9 @@
+# frozen_string_literal: true
 require 'gir_ffi_test_helper'
 
 describe GirFFI::Builders::FieldBuilder do
-  let(:instance) { GirFFI::Builders::FieldBuilder.new field_info }
+  let(:klass) { 'dummy' }
+  let(:instance) { GirFFI::Builders::FieldBuilder.new field_info, klass }
 
   describe 'for a field of type :gint8 with an offset' do
     let(:field_info) { get_field_introspection_data 'Regress', 'TestSimpleBoxedA', 'some_int8' }
@@ -9,9 +11,8 @@ describe GirFFI::Builders::FieldBuilder do
       expected = <<-CODE.reset_indentation
         def some_int8
           _v1 = @struct.to_ptr + #{field_info.offset}
-          _v2 = GirFFI::InOutPointer.new(:gint8, _v1)
-          _v3 = _v2.to_value
-          _v3
+          _v2 = _v1.get_int8(0)
+          _v2
         end
       CODE
       instance.getter_def.must_equal expected
@@ -21,9 +22,8 @@ describe GirFFI::Builders::FieldBuilder do
       expected = <<-CODE.reset_indentation
         def some_int8= value
           _v1 = @struct.to_ptr + #{field_info.offset}
-          _v2 = GirFFI::InOutPointer.new(:gint8, _v1)
-          _v3 = value
-          _v2.set_value _v3
+          _v2 = value
+          _v1.put_int8 0, _v2
         end
       CODE
       instance.setter_def.must_equal expected
@@ -36,14 +36,24 @@ describe GirFFI::Builders::FieldBuilder do
       expected = <<-CODE.reset_indentation
         def nested_a
           _v1 = @struct.to_ptr + #{field_info.offset}
-          _v2 = GirFFI::InOutPointer.new(Regress::TestSimpleBoxedA, _v1)
-          _v3 = _v2.to_value
-          _v4 = Regress::TestSimpleBoxedA.wrap(_v3)
-          _v4
+          _v2 = Regress::TestSimpleBoxedA.get_value_from_pointer(_v1, 0)
+          _v3 = Regress::TestSimpleBoxedA.wrap(_v2)
+          _v3
         end
       CODE
       instance.getter_def.must_equal expected
     end
+
+    it 'creates the right setter method' do
+      expected = <<-CODE.reset_indentation
+        def nested_a= value
+          _v1 = @struct.to_ptr + #{field_info.offset}
+          _v2 = Regress::TestSimpleBoxedA.copy_from(value)
+          Regress::TestSimpleBoxedA.copy_value_to_pointer(_v2, _v1)
+        end
+      CODE
+      instance.setter_def.must_equal expected
+    end
   end
 
   describe 'for a field of type :enum' do
@@ -52,9 +62,8 @@ describe GirFFI::Builders::FieldBuilder do
       expected = <<-CODE.reset_indentation
         def some_enum
           _v1 = @struct.to_ptr + #{field_info.offset}
-          _v2 = GirFFI::InOutPointer.new(Regress::TestEnum, _v1)
-          _v3 = _v2.to_value
-          _v3
+          _v2 = Regress::TestEnum.get_value_from_pointer(_v1, 0)
+          _v2
         end
       CODE
       instance.getter_def.must_equal expected
@@ -67,10 +76,9 @@ describe GirFFI::Builders::FieldBuilder do
       expected = <<-CODE.reset_indentation
         def some_union
           _v1 = @struct.to_ptr + #{field_info.offset}
-          _v2 = GirFFI::InOutPointer.new(:c, _v1)
-          _v3 = _v2.to_value
-          _v4 = GirFFI::SizedArray.wrap(Regress::TestStructE__some_union__union, 2, _v3)
-          _v4
+          _v2 = GirFFI::SizedArray.get_value_from_pointer(_v1, 0)
+          _v3 = GirFFI::SizedArray.wrap(Regress::TestStructE__some_union__union, 2, _v2)
+          _v3
         end
       CODE
       instance.getter_def.must_equal expected
@@ -80,10 +88,9 @@ describe GirFFI::Builders::FieldBuilder do
       expected = <<-CODE.reset_indentation
         def some_union= value
           _v1 = @struct.to_ptr + #{field_info.offset}
-          _v2 = GirFFI::InOutPointer.new(:c, _v1)
           GirFFI::ArgHelper.check_fixed_array_size 2, value, \"value\"
-          _v3 = GirFFI::SizedArray.from(Regress::TestStructE__some_union__union, 2, value)
-          _v2.set_value _v3
+          _v2 = GirFFI::SizedArray.copy_from(Regress::TestStructE__some_union__union, 2, value)
+          GirFFI::SizedArray.copy_value_to_pointer(_v2, _v1)
         end
       CODE
       instance.setter_def.must_equal expected
@@ -96,9 +103,8 @@ describe GirFFI::Builders::FieldBuilder do
       expected = <<-CODE.reset_indentation
         def class_init= value
           _v1 = @struct.to_ptr + #{field_info.offset}
-          _v2 = GirFFI::InOutPointer.new(GObject::ClassInitFunc, _v1)
-          _v3 = GObject::ClassInitFunc.from(value)
-          _v2.set_value _v3
+          _v2 = GObject::ClassInitFunc.from(value)
+          GObject::ClassInitFunc.copy_value_to_pointer(_v2, _v1)
         end
       CODE
       instance.setter_def.must_equal expected
@@ -114,13 +120,11 @@ describe GirFFI::Builders::FieldBuilder do
       expected = <<-CODE.reset_indentation
         def param_types
           _v1 = @struct.to_ptr + #{n_params_field_info.offset}
-          _v2 = GirFFI::InOutPointer.new(:guint32, _v1)
-          _v3 = _v2.to_value
-          _v4 = @struct.to_ptr + #{field_info.offset}
-          _v5 = GirFFI::InOutPointer.new([:pointer, :c], _v4)
-          _v6 = _v5.to_value
-          _v7 = GirFFI::SizedArray.wrap(:GType, _v3, _v6)
-          _v7
+          _v2 = _v1.get_uint32(0)
+          _v3 = @struct.to_ptr + #{field_info.offset}
+          _v4 = _v3.get_pointer(0)
+          _v5 = GirFFI::SizedArray.wrap(:GType, _v2, _v4)
+          _v5
         end
       CODE
       instance.getter_def.must_equal expected
