@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'gir_ffi_test_helper'
 
 describe GirFFI::Builders::ObjectBuilder do
@@ -9,6 +10,10 @@ describe GirFFI::Builders::ObjectBuilder do
     GirFFI::Builders::ObjectBuilder.new(
       get_introspection_data('Regress', 'TestSubObj'))
   end
+  let(:param_spec_builder) do
+    GirFFI::Builders::ObjectBuilder.new(
+      get_introspection_data('GObject', 'ParamSpec'))
+  end
 
   describe '#find_signal' do
     it 'finds the signal "test" for TestObj' do
@@ -27,14 +32,8 @@ describe GirFFI::Builders::ObjectBuilder do
       sig.name.must_equal 'changed'
     end
 
-    it "raises an error for a signal that doesn't exist" do
-      msg = nil
-      begin
-        obj_builder.find_signal 'foo'
-      rescue RuntimeError => e
-        msg = e.message
-      end
-      assert_match(/^Signal/, msg)
+    it "returns nil for a signal that doesn't exist" do
+      obj_builder.find_signal('foo').must_be_nil
     end
   end
 
@@ -49,10 +48,8 @@ describe GirFFI::Builders::ObjectBuilder do
       prop.name.must_equal 'int'
     end
 
-    it 'raises an error if the property is not found' do
-      proc do
-        sub_obj_builder.find_property('this-property-does-not-exist')
-      end.must_raise RuntimeError
+    it 'returns nil if the property is not found' do
+      sub_obj_builder.find_property('this-property-does-not-exist').must_be_nil
     end
   end
 
@@ -68,22 +65,50 @@ describe GirFFI::Builders::ObjectBuilder do
     end
   end
 
-  # TODO: Improve this spec to use less mocking
   describe 'for a struct without defined fields' do
+    let(:info) { get_introspection_data 'GObject', 'Binding' }
+
     it 'uses a single field of the parent struct type as the default layout' do
-      @gir = GObjectIntrospection::IRepository.default
-      @gir.require 'GObject', nil
+      info.n_fields.must_equal 0
 
-      allow(info = Object.new).to receive(:parent).and_return @gir.find_by_name 'GObject', 'Object'
-      allow(info).to receive(:fields).and_return []
-      allow(info).to receive(:info_type).and_return :object
-      allow(info).to receive(:safe_name).and_return 'Bar'
-      allow(info).to receive(:namespace).and_return 'Foo'
+      builder = GirFFI::Builders::ObjectBuilder.new info
 
-      @classbuilder = GirFFI::Builders::ObjectBuilder.new info
-
-      spec = @classbuilder.send :layout_specification
+      spec = builder.send :layout_specification
       assert_equal [:parent, GObject::Object::Struct, 0], spec
     end
   end
+
+  describe '#eligible_fields' do
+    it 'skips fields that have a matching getter method' do
+      result = param_spec_builder.eligible_fields
+      result.map(&:name).wont_include 'name'
+    end
+
+    it 'skips fields that have a matching property' do
+      result = obj_builder.eligible_fields
+      result.map(&:name).wont_include 'hash_table'
+    end
+
+    it 'skips the parent instance field' do
+      result = obj_builder.eligible_fields
+      result.map(&:name).wont_include 'parent_instance'
+    end
+  end
+
+  describe '#eligible_properties' do
+    let(:wi_builder) do
+      GirFFI::Builders::ObjectBuilder.new(
+        get_introspection_data('Regress', 'TestWi8021x'))
+    end
+
+    it 'includes properties that do not have a matching getter method' do
+      result = obj_builder.eligible_properties
+      result.map(&:name).must_include 'double'
+    end
+
+    it 'skips properties that have a matching getter method' do
+      result = wi_builder.eligible_properties
+      result.map(&:name).wont_include 'testbool'
+    end
+  end
 end
