@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'gir_ffi_test_helper'
 
 describe GirFFI::Builders::VFuncBuilder do
@@ -77,9 +78,9 @@ describe GirFFI::Builders::VFuncBuilder do
         def self.call_with_argument_mapping(_proc, _instance, arg, out)
           _v1 = GIMarshallingTests::Object.wrap(_instance)
           _v2 = arg
-          _v3 = GirFFI::InOutPointer.new(:gint8, out)
+          _v3 = out
           _v4 = _proc.call(_v1, _v2)
-          _v3.set_value _v4
+          _v3.put_int8 0, _v4
         end
         CODE
 
@@ -98,9 +99,29 @@ describe GirFFI::Builders::VFuncBuilder do
         def self.call_with_argument_mapping(_proc, _instance, arg, out)
           _v1 = GIMarshallingTests::Object.wrap(_instance)
           _v2 = arg
-          _v3 = GirFFI::InOutPointer.new(:gint8).tap { |ptr| out.put_pointer 0, ptr }
+          _v3 = FFI::MemoryPointer.new(:int8).tap { |ptr| out.put_pointer 0, ptr }
           _v4 = _proc.call(_v1, _v2)
-          _v3.set_value _v4
+          _v3.put_int8 0, _v4
+        end
+        CODE
+
+        result.must_equal expected
+      end
+    end
+
+    describe 'for a vfunc with a GObject::Value out argument allocated by them, the caller' do
+      let(:vfunc_info) do
+        get_vfunc_introspection_data('GIMarshallingTests', 'Object',
+                                     'vfunc_caller_allocated_out_parameter')
+      end
+
+      it 'returns a valid mapping method including receiver' do
+        expected = <<-CODE.reset_indentation
+        def self.call_with_argument_mapping(_proc, _instance, a)
+          _v1 = GIMarshallingTests::Object.wrap(_instance)
+          _v2 = a
+          _v3 = _proc.call(_v1)
+          GObject::Value.copy_value_to_pointer(GObject::Value.from(_v3), _v2)
         end
         CODE
 
@@ -119,11 +140,11 @@ describe GirFFI::Builders::VFuncBuilder do
         def self.call_with_argument_mapping(_proc, _instance, x, _error)
           _v1 = GIMarshallingTests::Object.wrap(_instance)
           _v2 = x
-          _v3 = GirFFI::InOutPointer.new([:pointer, :error], _error)
+          _v3 = _error
           begin
           _v4 = _proc.call(_v1, _v2)
           rescue => _v5
-          _v3.set_value GLib::Error.from(_v5)
+          _v3.put_pointer 0, GLib::Error.from(_v5)
           end
           return _v4
         end
@@ -143,7 +164,8 @@ describe GirFFI::Builders::VFuncBuilder do
         def self.call_with_argument_mapping(_proc, _instance)
           _v1 = GIMarshallingTests::Object.wrap(_instance)
           _v2 = _proc.call(_v1)
-          _v3 = GObject::Object.from(_v2.ref).to_ptr
+          _v2.ref
+          _v3 = GObject::Object.from(_v2).to_ptr
           return _v3
         end
         CODE
@@ -180,9 +202,10 @@ describe GirFFI::Builders::VFuncBuilder do
         expected = <<-CODE.reset_indentation
         def self.call_with_argument_mapping(_proc, _instance, object)
           _v1 = GIMarshallingTests::Object.wrap(_instance)
-          _v2 = GirFFI::InOutPointer.new([:pointer, GObject::Object], object)
+          _v2 = object
           _v3 = _proc.call(_v1)
-          _v2.set_value GObject::Object.from(_v3.ref)
+          _v3.ref
+          _v2.put_pointer 0, GObject::Object.from(_v3)
         end
         CODE
 
