@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'gir_ffi_test_helper'
 
 describe GirFFI::InfoExt::ITypeInfo do
@@ -290,16 +291,6 @@ describe GirFFI::InfoExt::ITypeInfo do
       end
     end
 
-    describe 'for a string' do
-      before do
-        allow(type_info).to receive(:tag).and_return :utf8
-      end
-
-      it 'returns an array containing :utf8' do
-        type_info.extra_conversion_arguments.must_equal [:utf8]
-      end
-    end
-
     describe 'for a fixed-size array' do
       before do
         allow(type_info).to receive(:tag).and_return :array
@@ -584,6 +575,15 @@ describe GirFFI::InfoExt::ITypeInfo do
       end
     end
 
+    describe 'for :void pointer' do
+      let(:tag) { :void }
+      let(:pointer?) { true }
+
+      it 'equals the none type' do
+        GObject.type_name(type_info.gtype).must_equal 'gpointer'
+      end
+    end
+
     describe 'for :gboolean' do
       let(:tag) { :gboolean }
       let(:pointer?) { false }
@@ -638,6 +638,15 @@ describe GirFFI::InfoExt::ITypeInfo do
       end
     end
 
+    describe 'for pointer to :glist' do
+      let(:tag) { :glist }
+      let(:pointer?) { true }
+
+      it 'equals the pointer type' do
+        GObject.type_name(type_info.gtype).must_equal 'gpointer'
+      end
+    end
+
     describe 'for arrays' do
       let(:tag) { :array }
       let(:pointer?) { true }
