@@ -1,4 +1,5 @@
 # coding: utf-8
+# frozen_string_literal: true
 require 'gir_ffi_test_helper'
 
 require 'gir_ffi'
@@ -44,6 +45,7 @@ describe GIMarshallingTests do
     it 'creates an instance using #new' do
       bx = GIMarshallingTests::BoxedStruct.new
       assert_instance_of GIMarshallingTests::BoxedStruct, bx
+      bx.struct.must_be :owned?
     end
 
     it 'has a working method #inv' do
@@ -55,22 +57,24 @@ describe GIMarshallingTests do
     it 'has a working function #inout' do
       bx = GIMarshallingTests::BoxedStruct.new
       bx.long_ = 42
+      bx.struct.must_be :owned?
 
       res = GIMarshallingTests::BoxedStruct.inout bx
-
-      # TODO: Deal with the fact that bx is now invalid (at least with
-      # gobject-introspection 1.40).
+      bx.struct.must_be :owned?
+      res.struct.must_be :owned?
 
       assert_equal 0, res.long_
     end
 
     it 'has a working function #out' do
       res = GIMarshallingTests::BoxedStruct.out
+      res.struct.must_be :owned?
       assert_equal 42, res.long_
     end
 
     it 'has a working function #returnv' do
       res = GIMarshallingTests::BoxedStruct.returnv
+      res.struct.must_be :owned?
       assert_equal 42, res.long_
       res.g_strv.must_be :==, %w(0 1 2)
     end
@@ -217,7 +221,7 @@ describe GIMarshallingTests do
       skip unless get_introspection_data 'GIMarshallingTests', 'Interface3'
       derived_klass.class_eval { include GIMarshallingTests::Interface3 }
       derived_instance = make_derived_instance do |info|
-        info.install_vfunc_implementation :test_variant_array_in, proc {|obj, in_|
+        info.install_vfunc_implementation :test_variant_array_in, proc { |obj, in_|
           obj.int = in_.to_a.first.get_byte
         }
       end
@@ -274,8 +278,9 @@ describe GIMarshallingTests do
     it 'has a writable field pointer' do
       instance.pointer.must_be_nil
       nested = GIMarshallingTests::NestedStruct.new
+      nested.simple_struct.int8 = 23
       instance.pointer = nested
-      instance.pointer.must_equal nested
+      instance.pointer.simple_struct.int8.must_equal 23
     end
   end
 
@@ -297,12 +302,12 @@ describe GIMarshallingTests do
 
     it 'has a working function #full_inout' do
       ob = GIMarshallingTests::Object.new 42
-      ref_count(ob).must_equal 1
+      ob.ref_count.must_equal 1
 
       res = GIMarshallingTests::Object.full_inout ob
 
-      ref_count(ob).must_be :>, 0
-      ref_count(res).must_equal 1
+      ob.ref_count.must_equal 1
+      res.ref_count.must_equal 1
 
       res.must_be_instance_of GIMarshallingTests::Object
       res.int.must_equal 0
@@ -310,33 +315,40 @@ describe GIMarshallingTests do
 
     it 'has a working function #full_out' do
       res = GIMarshallingTests::Object.full_out
-      assert_instance_of GIMarshallingTests::Object, res
+      res.must_be_instance_of GIMarshallingTests::Object
+      res.ref_count.must_equal 1
     end
 
     it 'has a working function #full_return' do
       res = GIMarshallingTests::Object.full_return
-      assert_instance_of GIMarshallingTests::Object, res
-    end
-
-    it 'has a working function #inout_same' do
-      skip 'This function is defined in the header but not implemented'
+      res.must_be_instance_of GIMarshallingTests::Object
+      res.ref_count.must_equal 1
     end
 
     it 'has a working function #none_inout' do
       ob = GIMarshallingTests::Object.new 42
+      ob.ref_count.must_equal 1
+
       res = GIMarshallingTests::Object.none_inout ob
-      assert_instance_of GIMarshallingTests::Object, res
-      ob.wont_equal res
+
+      ob.ref_count.must_equal 1
+      res.ref_count.must_equal 2
+
+      res.must_be_instance_of GIMarshallingTests::Object
+      ob.int.must_equal 42
+      res.int.must_equal 0
     end
 
     it 'has a working function #none_out' do
       res = GIMarshallingTests::Object.none_out
-      assert_instance_of GIMarshallingTests::Object, res
+      res.must_be_instance_of GIMarshallingTests::Object
+      res.ref_count.must_equal 2
     end
 
     it 'has a working function #none_return' do
       res = GIMarshallingTests::Object.none_return
-      assert_instance_of GIMarshallingTests::Object, res
+      res.must_be_instance_of GIMarshallingTests::Object
+      res.ref_count.must_equal 2
     end
 
     it 'has a working function #static_method' do
@@ -378,7 +390,7 @@ describe GIMarshallingTests do
                                                'vfunc_in_object_transfer_full')
       obj = nil
       derived_instance = make_derived_instance do |info|
-        info.install_vfunc_implementation :vfunc_in_object_transfer_full, proc {|_this, object|
+        info.install_vfunc_implementation :vfunc_in_object_transfer_full, proc { |_this, object|
           obj = object
         }
       end
@@ -393,13 +405,13 @@ describe GIMarshallingTests do
                                                'vfunc_in_object_transfer_none')
       obj = nil
       derived_instance = make_derived_instance do |info|
-        info.install_vfunc_implementation :vfunc_in_object_transfer_none, proc {|_this, object|
+        info.install_vfunc_implementation :vfunc_in_object_transfer_none, proc { |_this, object|
           obj = object
         }
       end
       result = derived_instance.
         get_ref_info_for_vfunc_in_object_transfer_none GIMarshallingTests::Object.gtype
-      ref_count(obj).must_be :>, 0
+      obj.ref_count.must_be :>, 0
       result.must_equal [2, false]
       obj.must_be_instance_of GIMarshallingTests::Object
     end
@@ -409,12 +421,12 @@ describe GIMarshallingTests do
                                                'vfunc_out_object_transfer_full')
       obj = nil
       derived_instance = make_derived_instance do |info|
-        info.install_vfunc_implementation :vfunc_out_object_transfer_full, proc {|_obj|
+        info.install_vfunc_implementation :vfunc_out_object_transfer_full, proc { |_obj|
           obj = GIMarshallingTests::Object.new(42)
         }
       end
       result = derived_instance.get_ref_info_for_vfunc_out_object_transfer_full
-      ref_count(obj).must_be :>, 0
+      obj.ref_count.must_be :>, 0
       # TODO: Check desired result
       result.must_equal [2, false]
     end
@@ -423,7 +435,7 @@ describe GIMarshallingTests do
       skip unless get_vfunc_introspection_data('GIMarshallingTests', 'Object',
                                                'vfunc_out_object_transfer_none')
       derived_instance = make_derived_instance do |info|
-        info.install_vfunc_implementation :vfunc_out_object_transfer_none, proc {|_obj|
+        info.install_vfunc_implementation :vfunc_out_object_transfer_none, proc { |_obj|
           GIMarshallingTests::Object.new 42
         }
       end
@@ -435,7 +447,7 @@ describe GIMarshallingTests do
       skip unless get_vfunc_introspection_data('GIMarshallingTests', 'Object',
                                                'vfunc_return_object_transfer_full')
       derived_instance = make_derived_instance do |info|
-        info.install_vfunc_implementation :vfunc_return_object_transfer_full, proc {|_obj|
+        info.install_vfunc_implementation :vfunc_return_object_transfer_full, proc { |_obj|
           GIMarshallingTests::Object.new 42
         }
       end
@@ -447,7 +459,7 @@ describe GIMarshallingTests do
       skip unless get_vfunc_introspection_data('GIMarshallingTests', 'Object',
                                                'vfunc_return_object_transfer_none')
       derived_instance = make_derived_instance do |info|
-        info.install_vfunc_implementation :vfunc_return_object_transfer_none, proc {|_obj|
+        info.install_vfunc_implementation :vfunc_return_object_transfer_none, proc { |_obj|
           GIMarshallingTests::Object.new 42
         }
       end
@@ -463,10 +475,15 @@ describe GIMarshallingTests do
       derived_instance.int.must_equal 23
     end
 
-    # TODO: Check if this is fixed
-    # See https://bugzilla.gnome.org/show_bug.cgi?id=727665
     it 'has a working method #int8_out' do
-      skip 'Introspection data is not generated correctly'
+      # GLib minor version and GObjectIntrospection minor version should be in sync
+      skip 'Introspection data is not generated correctly' if GLib::MINOR_VERSION < 42
+
+      derived_instance = make_derived_instance do |info|
+        info.install_vfunc_implementation :method_int8_out, proc { |_obj| 42 }
+      end
+
+      derived_instance.int8_out.must_equal 42
     end
 
     it 'has a working method #method' do
@@ -566,7 +583,7 @@ describe GIMarshallingTests do
       skip unless get_vfunc_introspection_data('GIMarshallingTests', 'Object',
                                                'vfunc_array_out_parameter')
       derived_instance = make_derived_instance do |info|
-        info.install_vfunc_implementation :vfunc_array_out_parameter, proc {|_obj|
+        info.install_vfunc_implementation :vfunc_array_out_parameter, proc { |_obj|
           [1.1, 2.2, 3.3]
         }
       end
@@ -582,7 +599,7 @@ describe GIMarshallingTests do
       skip unless get_vfunc_introspection_data('GIMarshallingTests', 'Object',
                                                'vfunc_caller_allocated_out_parameter')
       derived_instance = make_derived_instance do |info|
-        info.install_vfunc_implementation :vfunc_caller_allocated_out_parameter, proc {|_obj|
+        info.install_vfunc_implementation :vfunc_caller_allocated_out_parameter, proc { |_obj|
           'Hello!'
         }
       end
@@ -592,7 +609,7 @@ describe GIMarshallingTests do
 
     it 'has a working method #vfunc_meth_with_error' do
       derived_instance = make_derived_instance do |info|
-        info.install_vfunc_implementation :vfunc_meth_with_err, proc {|_object, x|
+        info.install_vfunc_implementation :vfunc_meth_with_err, proc { |_object, x|
           raise 'This is not the answer!' unless x == 42
           true
         }
@@ -730,11 +747,14 @@ describe GIMarshallingTests do
       result.must_equal 42
     end
 
-    it 'does not have field accessor methods' do
-      assert_raises(NoMethodError) { instance.parent_instance }
-      assert_raises(NoMethodError) { instance.parent_instance = nil }
-      assert_raises(NoMethodError) { instance.long_ }
-      assert_raises(NoMethodError) { instance.long_ = 1 }
+    it 'does not have accessors for its parent instance' do
+      instance.wont_respond_to :parent_instance
+      instance.wont_respond_to :parent_instance=
+    end
+
+    it 'has a readable field long_' do
+      instance.long_.must_equal 0
+      instance.wont_respond_to :long_=
     end
   end
 
@@ -1208,7 +1228,7 @@ describe GIMarshallingTests do
     it 'creates an instance using #new' do
       so = GIMarshallingTests::SubObject.new
       so.must_be_instance_of GIMarshallingTests::SubObject
-      GObject.type_name(GObject.type_from_instance so).
+      GObject.type_name_from_instance(so).
         must_equal 'GIMarshallingTestsSubObject'
     end
 
@@ -1224,9 +1244,9 @@ describe GIMarshallingTests do
       pass
     end
 
-    it 'does not have field accessors' do
-      assert_raises(NoMethodError) { instance.parent_instance }
-      assert_raises(NoMethodError) { instance.parent_instance = nil }
+    it 'does not have accessors for its parent instance' do
+      instance.wont_respond_to :parent_instance
+      instance.wont_respond_to :parent_instance=
     end
   end
 
@@ -1234,15 +1254,15 @@ describe GIMarshallingTests do
     it 'creates an instance using #new' do
       so = GIMarshallingTests::SubSubObject.new
       assert_instance_of GIMarshallingTests::SubSubObject, so
-      GObject.type_name(GObject.type_from_instance so).
+      GObject.type_name_from_instance(so).
         must_equal 'GIMarshallingTestsSubSubObject'
     end
 
     let(:instance) { GIMarshallingTests::SubSubObject.new }
 
-    it 'does not have field accessors' do
-      assert_raises(NoMethodError) { instance.parent_instance }
-      assert_raises(NoMethodError) { instance.parent_instance = nil }
+    it 'does not have accessors for its parent instance' do
+      instance.wont_respond_to :parent_instance
+      instance.wont_respond_to :parent_instance=
     end
   end
 
@@ -1450,12 +1470,14 @@ describe GIMarshallingTests do
     GIMarshallingTests.array_struct_in arr
   end
 
-  # NOTE: Should be run with valgrind. See gimarhallingtests.c.
   it 'has a working function #array_struct_take_in' do
     arr = [1, 2, 3].map do |val|
       GIMarshallingTests::BoxedStruct.new.tap { |struct| struct.long_ = val }
     end
+    # NOTE: This copies values so arr's elements stay valid
     GIMarshallingTests.array_struct_take_in arr
+
+    arr.map(&:long_).must_equal [1, 2, 3]
   end
 
   it 'has a working function #array_struct_value_in' do
@@ -1820,8 +1842,9 @@ describe GIMarshallingTests do
   it 'has a working function #gclosure_return' do
     cl = GIMarshallingTests.gclosure_return
     gv = GObject::Value.wrap_ruby_value 0
-    cl.invoke gv, nil, nil
+    result = cl.invoke gv, []
     assert_equal 42, gv.get_value
+    result.must_equal 42
   end
 
   it 'has a working function #genum_in' do
@@ -2588,12 +2611,13 @@ describe GIMarshallingTests do
     res.must_be_instance_of GIMarshallingTests::OverridesStruct
   end
 
-  # TODO: Wait for fixed version of param_spec_in_bool to land in Debian/Ubuntu
-  # See https://bugzilla.gnome.org/show_bug.cgi?id=728409
   it 'has a working function #param_spec_in_bool' do
-    skip 'param_spec_in_bool tests the wrong type'
+    # GLib minor version and GObjectIntrospection minor version should be in sync
+    skip 'param_spec_in_bool tests the wrong type' if GLib::MINOR_VERSION < 42
+
     ps = GObject.param_spec_boolean 'mybool', 'nick', 'blurb', true, :readable
     GIMarshallingTests.param_spec_in_bool ps
+    pass
   end
 
   it 'has a working function #param_spec_out' do
