@@ -1,12 +1,15 @@
+# frozen_string_literal: true
 require 'gir_ffi_test_helper'
 
 GirFFI.setup :Gio
 
 # Tests generated methods and functions in the Gio namespace.
 describe 'The generated Gio module' do
-  describe '#file_new_for_path, a method returning an interface,' do
+  cover 'Gio'
+
+  describe 'File#new_for_path, a method returning an interface,' do
     it 'returns an object of a more specific class' do
-      file = Gio.file_new_for_path('/')
+      file = Gio::File.new_for_path('/')
       refute_instance_of Gio::File, file
       assert_includes file.class.ancestors, Gio::File
     end
@@ -109,7 +112,7 @@ describe 'The generated Gio module' do
     end
 
     it 'cannot write the property "state-type" with #state_type=' do
-      proc { simple_action.state_type = nil }.must_raise NoMethodError
+      simple_action.wont_respond_to :state_type=
     end
   end
 end
