@@ -1,8 +1,11 @@
+# frozen_string_literal: true
 require 'gir_ffi_test_helper'
 
 # Tests generated methods and functions in the GTop namespace. This namespace
 # contains types with bad names, like 'glibtop_cpu'.
 describe 'The generated GTop module' do
+  cover 'GTop'
+
   before do
     begin
       GirFFI.setup :GTop
