@@ -1,4 +1,5 @@
 # coding: utf-8
+# frozen_string_literal: true
 require 'gir_ffi_test_helper'
 
 GirFFI.setup :PangoFT2
@@ -6,7 +7,7 @@ GirFFI.setup :PangoFT2
 # Tests generated methods and functions in the PangoFT2 namespace.
 describe PangoFT2 do
   describe PangoFT2::FontMap do
-    it "has a working method #load_font" do
+    it 'has a working method #load_font' do
       font_map = PangoFT2::FontMap.new
       context = font_map.create_context
       font_description = Pango::FontDescription.new
@@ -14,4 +15,3 @@ describe PangoFT2 do
     end
   end
 end
-
