@@ -1,4 +1,5 @@
 # coding: utf-8
+# frozen_string_literal: true
 require 'gir_ffi_test_helper'
 
 GirFFI.setup :Pango
@@ -6,11 +7,11 @@ GirFFI.setup :Pango
 # Tests generated methods and functions in the Pango namespace.
 describe Pango do
   describe Pango::Language do
-    it "has a working method #get_scripts" do
+    it 'has a working method #get_scripts' do
       lang = Pango::Language.from_string 'ja'
       result = lang.get_scripts
 
-      if GirFFI::SizedArray === result
+      if result.is_a? GirFFI::SizedArray
         scripts = result
       else
         ptr, size = *result
