@@ -1,6 +1,9 @@
+# frozen_string_literal: true
 require 'gir_ffi_test_helper'
 
 describe 'The generated WarnLib module' do
+  cover 'WarnLib'
+
   before do
     begin
       GirFFI.setup :WarnLib
